from pythran.tests import TestEnv
import builtins
import unittest

exceptions = [i for i in dir(builtins)
              if (isinstance(getattr(builtins, i), type) and
                  issubclass(getattr(builtins, i),
                             builtins.BaseException))]

exception_args = {
    "BaseException": "('abc')",
    "SystemExit": "('a','b','c')",
    "KeyboardInterrupt": "('a','b','c')",
    "GeneratorExit": "('a','b','c')",
    "Exception": "('a','b','c')",
    "StopIteration": "('a','b','c')",
    "Warning": "('a','b','c')",
    "BytesWarning": "('a','b','c')",
    "UnicodeWarning": "('a','b','c')",
    "ImportWarning": "('a','b','c')",
    "FutureWarning": "('a','b','c')",
    "UserWarning": "('a','b','c')",
    "SyntaxWarning": "('a','b','c')",
    "RuntimeWarning": "('a','b','c')",
    "PendingDeprecationWarning": "('a','b','c')",
    "DeprecationWarning": "('a','b','c')",
    "BufferError": "('a','b','c')",
    "ArithmeticError": "('a','b','c')",
    "AssertionError": "('a','b','c')",
    "AttributeError": "('a','b','c')",
    "EnvironmentError": "('a','b','c','d')",
    "EOFError": "('a','b','c')",
    "ImportError": "('a','b','c')",
    "LookupError": "('a','b','c')",
    "MemoryError": "('a','b','c')",
    "NameError": "('a','b','c')",
    "ReferenceError": "('a','b','c')",
    "RuntimeError": "('a','b','c')",
    "SyntaxError": "('a','b','c')",
    "SystemError": "('a','b','c')",
    "TypeError": "('a','b','c')",
    "ValueError": "('a','b','c')",
    "FloatingPointError": "('a','b','c')",
    "OverflowError": "('a','b','c')",
    "ZeroDivisionError": "('a','b','c')",
    "IOError": "('a','b','c')",
    "OSError": "('a','b','c')",
    "IndexError": "('a','b','c')",
    "KeyError": "('a','b','c')",
    "UnboundLocalError": "('a','b','c')",
    "NotImplementedError": "('a','b','c')",
    "IndentationError": "('a','b','c')",
    "TabError": "('a','b','c')",
    "UnicodeError": "('a','b','c')",
}

class TestException(TestEnv):

    def test_BaseException(self):
        self.run_test("def BaseException_():\n try: raise BaseException('a','b','c')\n except BaseException as e: return e.args", BaseException_=[])

    def test_SystemExit(self):
        self.run_test("def SystemExit_():\n try: raise SystemExit('a','b','c')\n except SystemExit as e: return e.args", SystemExit_=[])

    def test_KeyboardInterrupt(self):
        self.run_test("def KeyboardInterrupt_():\n try: raise KeyboardInterrupt('a','b','c')\n except KeyboardInterrupt as e: return e.args", KeyboardInterrupt_=[])

    def test_GeneratorExit(self):
        self.run_test("def GeneratorExit_():\n try: raise GeneratorExit('a','b','c')\n except GeneratorExit as e: return e.args", GeneratorExit_=[])

    def test_Exception(self):
        self.run_test("def Exception_():\n try: raise Exception('a','b','c')\n except Exception as e: return e.args", Exception_=[])

    def test_StopIteration(self):
        self.run_test("def StopIteration_():\n try: raise StopIteration('a','b','c')\n except StopIteration as e: return e.args", StopIteration_=[])

    def test_Warning(self):
        self.run_test("def Warning_():\n try: raise Warning('a','b','c')\n except Warning as e: return e.args", Warning_=[])

    def test_BytesWarning(self):
        self.run_test("def BytesWarning_():\n try: raise BytesWarning('a','b','c')\n except BytesWarning as e: return e.args", BytesWarning_=[])

    def test_UnicodeWarning(self):
        self.run_test("def UnicodeWarning_():\n try: raise UnicodeWarning('a','b','c')\n except UnicodeWarning as e: return e.args", UnicodeWarning_=[])

    def test_ImportWarning(self):
        self.run_test("def ImportWarning_():\n try: raise ImportWarning('a','b','c')\n except ImportWarning as e: return e.args", ImportWarning_=[])

    def test_FutureWarning(self):
        self.run_test("def FutureWarning_():\n try: raise FutureWarning('a','b','c')\n except FutureWarning as e: return e.args", FutureWarning_=[])

    def test_UserWarning(self):
        self.run_test("def UserWarning_():\n try: raise UserWarning('a','b','c')\n except UserWarning as e: return e.args", UserWarning_=[])

    def test_SyntaxWarning(self):
        self.run_test("def SyntaxWarning_():\n try: raise SyntaxWarning('a','b','c')\n except SyntaxWarning as e: return e.args", SyntaxWarning_=[])

    def test_RuntimeWarning(self):
        self.run_test("def RuntimeWarning_():\n try: raise RuntimeWarning('a','b','c')\n except RuntimeWarning as e: return e.args", RuntimeWarning_=[])

    def test_PendingDeprecationWarning(self):
        self.run_test("def PendingDeprecationWarning_():\n try: raise PendingDeprecationWarning('a','b','c')\n except PendingDeprecationWarning as e: return e.args", PendingDeprecationWarning_=[])

    def test_DeprecationWarning(self):
        self.run_test("def DeprecationWarning_():\n try: raise DeprecationWarning('a','b','c')\n except DeprecationWarning as e: return e.args", DeprecationWarning_=[])

    def test_BufferError(self):
        self.run_test("def BufferError_():\n try: raise BufferError('a','b','c')\n except BufferError as e: return e.args", BufferError_=[])

    def test_ArithmeticError(self):
        self.run_test("def ArithmeticError_():\n try: raise ArithmeticError('a','b','c')\n except ArithmeticError as e: return e.args", ArithmeticError_=[])

    @unittest.skip("incompatible with py.test")
    def test_AssertionError(self):
        self.run_test("def AssertionError_():\n try: raise AssertionError('a','b','c')\n except AssertionError as e: return e.args", AssertionError_=[])

    def test_AttributeError(self):
        self.run_test("def AttributeError_():\n try: raise AttributeError('a','b','c')\n except AttributeError as e: return e.args", AttributeError_=[])

    def test_EnvironmentError2(self):
        self.run_test("def EnvironmentError2_():\n try: raise EnvironmentError('a','b')\n except EnvironmentError as e: return e.args", EnvironmentError2_=[])

    def test_EnvironmentError1(self):
        self.run_test("def EnvironmentError1_():\n try: raise EnvironmentError('a')\n except EnvironmentError as e: return e.args", EnvironmentError1_=[])

    def test_EOFError(self):
        self.run_test("def EOFError_():\n try: raise EOFError('a','b','c')\n except EOFError as e: return e.args", EOFError_=[])

    def test_ImportError(self):
        self.run_test("def ImportError_():\n try: raise ImportError('a','b','c')\n except ImportError as e: return e.args", ImportError_=[])

    def test_LookupError(self):
        self.run_test("def LookupError_():\n try: raise LookupError('a','b','c')\n except LookupError as e: return e.args", LookupError_=[])

    def test_MemoryError(self):
        self.run_test("def MemoryError_():\n try: raise MemoryError('a','b','c')\n except MemoryError as e: return e.args", MemoryError_=[])

    def test_NameError(self):
        self.run_test("def NameError_():\n try: raise NameError('a','b','c')\n except NameError as e: return e.args", NameError_=[])

    def test_ReferenceError(self):
        self.run_test("def ReferenceError_():\n try: raise ReferenceError('a','b','c')\n except ReferenceError as e: return e.args", ReferenceError_=[])

    def test_RuntimeError(self):
        self.run_test("def RuntimeError_():\n try: raise RuntimeError('a','b','c')\n except RuntimeError as e: return e.args", RuntimeError_=[])

    def test_SyntaxError(self):
        self.run_test("def SyntaxError_():\n try: raise SyntaxError('a','b','c')\n except SyntaxError as e: return e.args", SyntaxError_=[])

    def test_SystemError(self):
        self.run_test("def SystemError_():\n try: raise SystemError('a','b','c')\n except SystemError as e: return e.args", SystemError_=[])

    def test_TypeError(self):
        self.run_test("def TypeError_():\n try: raise TypeError('a','b','c')\n except TypeError as e: return e.args", TypeError_=[])

    def test_ValueError(self):
        self.run_test("def ValueError_():\n try: raise ValueError('a','b','c')\n except ValueError as e: return e.args", ValueError_=[])

    def test_FloatingPointError(self):
        self.run_test("def FloatingPointError_():\n try: raise FloatingPointError('a','b','c')\n except FloatingPointError as e: return e.args", FloatingPointError_=[])

    def test_OverflowError(self):
        self.run_test("def OverflowError_():\n try: raise OverflowError('a','b','c')\n except OverflowError as e: return e.args", OverflowError_=[])

    def test_ZeroDivisionError(self):
        self.run_test("def ZeroDivisionError_():\n try: raise ZeroDivisionError('a','b','c')\n except ZeroDivisionError as e:\n  return e.args", ZeroDivisionError_=[])

    def test_IOError(self):
        self.run_test("def IOError_():\n try: raise IOError('a','b','c')\n except IOError as e: return e.args", IOError_=[])

    def test_OSError(self):
        self.run_test("def OSError_():\n try: raise OSError('a','b','c')\n except OSError as e: return e.args", OSError_=[])

    def test_IndexError(self):
        self.run_test("def IndexError_():\n try: raise IndexError('a','b','c')\n except IndexError as e: return e.args", IndexError_=[])

    def test_KeyError(self):
        self.run_test("def KeyError_():\n try: raise KeyError('a','b','c')\n except KeyError as e: return e.args", KeyError_=[])

    def test_UnboundLocalError(self):
        self.run_test("def UnboundLocalError_():\n try: raise UnboundLocalError('a','b','c')\n except UnboundLocalError as e: return e.args", UnboundLocalError_=[])

    def test_NotImplementedError(self):
        self.run_test("def NotImplementedError_():\n try: raise NotImplementedError('a','b','c')\n except NotImplementedError as e: return e.args", NotImplementedError_=[])

    def test_IndentationError(self):
        self.run_test("def IndentationError_():\n try: raise IndentationError('a','b','c')\n except IndentationError as e: return e.args", IndentationError_=[])

    def test_TabError(self):
        self.run_test("def TabError_():\n try: raise TabError('a','b','c')\n except TabError as e: return e.args", TabError_=[])

    def test_UnicodeError(self):
        self.run_test("def UnicodeError_():\n try: raise UnicodeError('a','b','c')\n except UnicodeError as e: return e.args", UnicodeError_=[])

    def test_multiple_exception(self):
        self.run_test("def multiple_exception_():\n try:\n  raise OverflowError('a','b','c')\n except IOError as e:\n  a=2 ; print(a) ; return e.args\n except OverflowError as e:\n  return e.args", multiple_exception_=[])

    def test_multiple_tuple_exception(self):
        self.run_test("def multiple_tuple_exception_():\n try:\n  raise OverflowError('a','b','c')\n except (IOError, OSError):\n  a=3;print(a)\n except OverflowError as e:\n  return e.args", multiple_tuple_exception_=[])

    def test_reraise_exception(self):
        self.run_test("def reraise_exception_():\n try:\n  raise OverflowError('a','b','c')\n except IOError:\n  raise\n except:  return 'ok'", reraise_exception_=[])

    def test_else2_exception(self):
        self.run_test("def else2_exception_():\n try:\n  raise 1\n  return 0,'bad'\n except:\n  a=2\n else:\n  return 0,'bad2'\n return a,'ok'", else2_exception_=[])

    def test_else_exception(self):
        self.run_test("def else_exception_():\n try:\n  a=2\n except:\n  return 0,'bad'\n else:\n  return a,'ok'\n return 0,'bad2'", else_exception_=[])

    def test_enverror_exception(self):
        self.run_test("def enverror_exception_():\n try:\n  raise EnvironmentError('a','b','c')\n except EnvironmentError as e:\n  return (e.errno,e.strerror,e.filename)", enverror_exception_=[])

    def test_finally_exception(self):
        self.run_test("def finally_exception_():\n try:\n  a=2\n except:\n  return 0,'bad'\n finally:\n  return a,'good'", finally_exception_=[])

    def test_finally2_exception(self):
        self.run_test("def finally2_exception_():\n try:\n  a=1\n except:\n  a=2\n finally:\n  return a,'good'", finally2_exception_=[])

    def test_str1_exception(self):
        self.run_test("def str1_exception_():\n try:\n  raise EnvironmentError('a')\n except EnvironmentError as e:\n  return str(e)", str1_exception_=[])

    def test_str2_exception(self):
        self.run_test("def str2_exception_():\n try:\n  raise EnvironmentError('a','b')\n except EnvironmentError as e:\n  return str(e)", str2_exception_=[])

    def test_str3_exception(self):
        self.run_test("def str3_exception_():\n try:\n  raise EnvironmentError('a','b','c')\n except EnvironmentError as e:\n  return str(e)", str3_exception_=[])

    def test_no_msg_exception(self):
        self.run_test("def no_msg_exception_():\n try: raise IndexError()\n except IndexError as e: return e.args", no_msg_exception_=[])

# test if exception translators are registered in pythran

    def test_EnvironmentError3_register(self):
        self.run_test("def EnvironmentError3_register():\n raise EnvironmentError('a','b','c')", EnvironmentError3_register=[], check_exception=True)

    def test_EnvironmentError2_register(self):
        self.run_test("def EnvironmentError2_register():\n raise EnvironmentError('a','b')", EnvironmentError2_register=[], check_exception=True)

    def test_EnvironmentError1_register(self):
        self.run_test("def EnvironmentError1_register():\n raise EnvironmentError('a')", EnvironmentError1_register=[], check_exception=True)

    def test_multiple_exception_register(self):
        self.run_test("def multiple_exception_register():\n raise OverflowError('a','b','c')", multiple_exception_register=[], check_exception=True)

    def test_multiple_tuple_exception_register(self):
        self.run_test("def multiple_tuple_exception_register():\n raise OverflowError('a','b','c')", multiple_tuple_exception_register=[], check_exception=True)

    def test_reraise_exception_register(self):
        self.run_test("def reraise_exception_register():\n raise OverflowError('a','b','c')", reraise_exception_register=[], check_exception=True)

    def test_enverror_exception_register(self):
        self.run_test("def enverror_exception_register():\n raise EnvironmentError('a','b','c')", enverror_exception_register=[], check_exception=True)

    def test_str1_exception_register(self):
        self.run_test("def str1_exception_register():\n raise EnvironmentError('a')", str1_exception_register=[], check_exception=True)

    def test_str2_exception_register(self):
        self.run_test("def str2_exception_register():\n raise EnvironmentError('a','b')", str2_exception_register=[], check_exception=True)

    def test_str3_exception_register(self):
        self.run_test("def str3_exception_register():\n raise EnvironmentError('a','b','c')", str3_exception_register=[], check_exception=True)

    def test_str4_exception_register(self):
        self.run_test("def str4_exception_register():\n raise EnvironmentError('a','b','c','d')", str4_exception_register=[], check_exception=True)

    def test_str5_exception_register(self):
        self.run_test("def str5_exception_register():\n raise EnvironmentError('a','b','c','d','e')", str5_exception_register=[], check_exception=True)

    def test_no_msg_exception_register(self):
        self.run_test("def no_msg_exception_register():\n raise IndexError()", no_msg_exception_register=[], check_exception=True)

for exception in exceptions:
    # This one is not compatible with pytest
    if str(exception) in ("AssertionError", "UnicodeDecodeError",
                          "UnicodeEncodeError", "UnicodeTranslateError"):
        continue
    if exception not in exception_args:
        continue
    args = exception_args[exception]
    code = 'def {exception}_register(): raise {exception}{args}'.format(**locals())
    setattr(TestException, 'test_' + str(exception) + "_register",
            eval("""lambda self: self.run_test('''{0}''', {1}_register=[], check_exception=True)""".format(code, exception)))




#runas solve(1000)
#pythran export solve(int)
'''
What is the first term in the Fibonacci sequence to contain 1000 digits
'''

import math
def solve(digit):
 phi = (1 + pow(5, 0.5)) / 2
 c = math.log10(5) / 2
 logphi = math.log10(phi)
 n = 1
 while True:
     if n * logphi - c >= digit - 1:
         return n
         break
     n = n + 1



#from http://stackoverflow.com/questions/26823312/numba-or-cython-acceleration-in-reaction-diffusion-algorithm
#runas GrayScott(40, 0.16, 0.08, 0.04, 0.06)
#pythran export GrayScott(int, float, float, float, float)
import numpy as np
def GrayScott(counts, Du, Dv, F, k):
    n = 300
    U = np.zeros((n+2,n+2), dtype=np.float32)
    V = np.zeros((n+2,n+2), dtype=np.float32)
    u, v = U[1:-1,1:-1], V[1:-1,1:-1]

    r = 20
    u[:] = 1.0
    nd2 = int(n/2)
    U[nd2-r:nd2+r,nd2-r:nd2+r] = 0.50
    V[nd2-r:nd2+r,nd2-r:nd2+r] = 0.25
    # commented out because non-reproductible
    if 0:
        u += 0.15*np.random.random((n,n))
        v += 0.15*np.random.random((n,n))
    else:
        u += 0.15*np.ones((n,n))
        v += 0.15*np.ones((n,n))

    for i in range(counts):
        Lu = (                 U[0:-2,1:-1] +
              U[1:-1,0:-2] - 4*U[1:-1,1:-1] + U[1:-1,2:] +
                               U[2:  ,1:-1] )
        Lv = (                 V[0:-2,1:-1] +
              V[1:-1,0:-2] - 4*V[1:-1,1:-1] + V[1:-1,2:] +
                               V[2:  ,1:-1] )
        uvv = u*v*v
        u += Du*Lu - uvv + F*(1 - u)
        v += Dv*Lv + uvv - (F + k)*v

    return V



#from http://jakascorner.com/blog/2016/08/omp-cancel.html

def has_zero(matrix):
    has_zero = False
    rows, cols = matrix.shape

    #omp parallel default(none) shared(matrix, has_zero)
    #omp for
    for row in range(rows):
        for col in range(cols):
            if matrix[row, col] == 0:
                #omp critical
                has_zero = True
                #omp cancel for

    return has_zero

import numpy as np
def cancel_for():
    data = np.array([[1,2,3], [4,0,5], [7, 8, 9]])
    res = has_zero(data)
    return res



#pythran export collatz_modified(int)
#runas collatz_modified(10)

def collatz_modified(target):
    start = 1
    while True:
        i = start
        steps = 0
        while True:
            if i == 1:
                break
            if i % 2 == 0:
                i = i // 2
            else:
                i = 3 * i + 1
            steps += 1
        if steps == target:
            return start
        start += 1



def omp_barrier():
    import omp
    from time import sleep
    result1 = 0
    result2 = 0
    #omp parallel num_threads(4)
    if 1:
        use_omp = omp.in_parallel()
        rank = omp.get_thread_num()
        if rank == 1:
            sleep(0.5)
            result2 = 3
        #omp barrier
        if rank == 2:
            result1 = result2
    if use_omp:
        return result1 == 3
    else:
        return result1 == 0



#runas solve(5)
#pythran export solve(int)

'''
Surprisingly there are only three numbers that can be written as the sum of
fourth powers of their digits:

1634 = 1^4 + 6^4 + 3^4 + 4^4
8208 = 8^4 + 2^4 + 0^4 + 8^4
9474 = 9^4 + 4^4 + 7^4 + 4^4
As 1 = 1^4 is not a sum it is not included.

The sum of these numbers is 1634 + 8208 + 9474 = 19316.

Find the sum of all the numbers that can be written as the sum of fifth powers
of their digits.
'''

def power_of_digits(n, p):
    s = 0
    while n > 0:
        d = n % 10
        n = n // 10
        s = s + pow(d, p)
    return s


def solve(p):
    return sum(n for n in range(2, 200000) if power_of_digits(n, p) == n)



#pythran export test()
#norunas test() because of random input
#bench test()
import  math, random
from functools import reduce

# a point is a tuple
# a cluster is a list of tuple and a point (the centroid)

def calculateCentroid(cluster):
    reduce_coord = lambda i: reduce(lambda x,p : x + p[i], cluster,0.0)
    centroid_coords = [reduce_coord(i)/len(cluster) for i in range(len(cluster[0]))]
    return centroid_coords

def kmeans(points, k, cutoff):
    initial = random.sample(points, k)
    clusters = [[p] for p in initial]
    centroids = [ calculateCentroid(c) for c in clusters ]
    while True:
        lists = [ [] for c in clusters]
        for p in points:
            smallest_distance = getDistance(p,centroids[0])
            index = 0
            for i in range(len(clusters[1:])):
                distance = getDistance(p, centroids[i+1])
                if distance < smallest_distance:
                    smallest_distance = distance
                    index = i+1
            lists[index].append(p)
        biggest_shift = 0.0
        for i in range(len(clusters)):
            if lists[i]:
                new_cluster, new_centroid = (lists[i], calculateCentroid(lists[i]))
                shift = getDistance(centroids[i], new_centroid)
                clusters[i] = new_cluster
                centroids[i] = new_centroid
                biggest_shift = max(biggest_shift, shift)
        if biggest_shift < cutoff:
            break
    return clusters

def getDistance(a, b):
    ret = reduce(lambda x,y: x + pow((a[y]-b[y]), 2),range(len(a)),0.0)
    return math.sqrt(ret)

def makeRandomPoint(n, lower, upper):
    return [random.uniform(lower, upper) for i in range(n)]

def test():
    num_points, dim, k, cutoff, lower, upper = 500, 10, 50, 0.001, 0, 2000
    points = [ makeRandomPoint(dim, lower, upper) for i in range(num_points) ]
    clusters = kmeans(points, k, cutoff)
    #for c in clusters:
    #    print(c)
    return clusters



def omp_parallel_shared():
    sum = 0
    LOOPCOUNT = 1000
    known_sum = (LOOPCOUNT * (LOOPCOUNT + 1)) / 2
    if 'omp parallel shared(sum)':
        mysum = 0
        'omp for'
        for i in range(1, LOOPCOUNT + 1):
            mysum += i
        if 'omp critical':
            sum += mysum

    return known_sum == sum



"""Module to check coding style."""
import glob
import os
import subprocess
import unittest

excluded_files = ['pdqsort.hpp']

def test_clang_format():
    """Check coding style for cpp files."""
    clang_format = 'clang-format-3.6'
    try:
        subprocess.check_call([clang_format, '--version'])
    except OSError:
        raise unittest.SkipTest(clang_format + " not available")

    pythran_dir = os.path.abspath(os.path.dirname(os.path.dirname(__file__)))
    for root, _, _ in os.walk(os.path.join(pythran_dir, "pythonic")):
        for file_ in glob.glob(os.path.join(root, "*.hpp")):
            if os.path.basename(file_) in excluded_files:
                continue
            cmd = "{cmd} {file} | diff -u {file} -".format(
                cmd=clang_format,
                file=os.path.join(root, file_))
            process = subprocess.Popen(cmd, shell=True, stdout=subprocess.PIPE, universal_newlines=True)
            assert not process.wait(), process.stdout.read()



#from http://rosettacode.org/wiki/Roman_numerals/Decode#Python
#runas decode('MCMXC')
#runas decode('MMVIII')
#runas decode('MDCLXVI')
#pythran export decode(str)

def decode( roman ):
    s, t = 'MDCLXVI', (1000, 500, 100, 50, 10, 5, 1)
    _rdecode = dict(zip(s, t))
    result = 0
    for r, r1 in zip(roman, roman[1:]):
        rd, rd1 = _rdecode[r], _rdecode[r1]
        result += -rd if rd < rd1 else rd
    return result + _rdecode[roman[-1]]



import numpy as np
def collapse_bug(x, m, btx):
    n = int(np.log2(m))
    lv = np.zeros((x.shape[0], n))
    N = x.shape[0]
    #omp parallel for collapse(2)
    for i in range(N):
        for k in range(n):
            lv[i,k] = abs(x[i]- btx[k])
    return lv

def pythran_collapse():
    from random import randint
    n = randint(50, 50)
    x = np.ones(n)
    return collapse_bug(x, 5, 3. * x)



from .method_in_imported_module import aa, hh
from .other_method_in_imported_module import CC, DD

def cc():
    return aa(3)



#pythran export dd(int)
def dd(o):
    return hh(o)


XX = cc() + [3]
YY = CC()

#pythran export bb()
def bb():
    return XX, YY, DD()



# from http://rosettacode.org/wiki/Yin_and_yang#Python
#pythran export yinyang(int)
#runas yinyang(4)

import math
def yinyang(n=3):
        radii   = [i * n for i in [1, 3, 6]]
        ranges  = [list(range(-r, r+1)) for r in radii]
        squares = [[ (x,y) for x in rnge for y in rnge]
                   for rnge in ranges]
        circles = [[ (x,y) for x,y in sqrpoints
                     if math.hypot(x,y) <= radius ]
                   for sqrpoints, radius in zip(squares, radii)]
        m = {(x,y):' ' for x,y in squares[-1]}
        for x,y in circles[-1]:
                m[x,y] = '*'
        for x,y in circles[-1]:
                if x>0: m[(x,y)] = '.'
        for x,y in circles[-2]:
                m[(x,y+3*n)] = '*'
                m[(x,y-3*n)] = '.'
        for x,y in circles[-3]:
                m[(x,y+3*n)] = '.'
                m[(x,y-3*n)] = '*'
        return '\n'.join(''.join(m[(x,y)] for x in reversed(ranges[-1])) for y in ranges[-1])



#from http://stackoverflow.com/questions/7741878/how-to-apply-numpy-linalg-norm-to-each-row-of-a-matrix/7741976#7741976
#pythran export l2_norm(float64[][])
#runas import numpy as np ; N = 100 ; x = np.arange(N*N, dtype=np.float64).reshape((N,N)) ; l2_norm(x)
#bench import numpy as np ; N = 10000 ; x = np.arange(N*N, dtype=np.float64).reshape((N,N)) ; l2_norm(x)
import numpy as np
def l2_norm(x):
    return np.sqrt(np.sum(np.abs(x)**2, 1))



def omp_single():
    nr_threads_in_single = 0
    result = 0
    nr_iterations = 0
    LOOPCOUNT = 1000

    if 'omp parallel':
        for i in range(LOOPCOUNT):
            if 'omp single':
                'omp flush'
                nr_threads_in_single += 1
                'omp flush'
                nr_iterations += 1
                nr_threads_in_single -= 1
                result += nr_threads_in_single
    return result == 0 and nr_iterations == LOOPCOUNT



#pythran export times(int or str, int)
#runas times(200, 3)
#runas times('200', 3)

def times(n, m):
    return n*m

#pythran export check([str or float] list, int)
#runas check(['1'], 2)
#runas check([1.5], 2)

def check(x, y):
    if y:
        return x, y
    else:
        return x + x, y

#pythran export check2(str or float list, str or bool list)
#runas check2('1', '2')
#runas check2('1', [True])
#runas check2([1.5], 'True')
#runas check2([1.5], [False])

def check2(x, y):
    if y:
        return x, y
    else:
        return x + x, y



import unittest
from pythran.tests import TestEnv
import numpy
from pythran.typing import NDArray


@TestEnv.module
class TestNumpyLinalg(TestEnv):

    def test_linalg_norm0(self):
        self.run_test("def linalg_norm0(x): from numpy.linalg import norm ; return norm(x)", numpy.arange(6.), linalg_norm0=[NDArray[float,:]])

    def test_linalg_norm1(self):
        self.run_test("def linalg_norm1(x): from numpy.linalg import norm ; return norm(x)", numpy.arange(6.).reshape(2,3), linalg_norm1=[NDArray[float,:,:]])

    def test_linalg_norm2(self):
        self.run_test("def linalg_norm2(x): from numpy.linalg import norm ; from numpy import inf ; return norm(x, inf)", numpy.arange(6.), linalg_norm2=[NDArray[float,:]])

    def test_linalg_norm3(self):
        self.run_test("def linalg_norm3(x): from numpy.linalg import norm ; from numpy import inf ; return norm(x, -inf)", numpy.arange(6.), linalg_norm3=[NDArray[float,:]])

    def test_linalg_norm4(self):
        self.run_test("def linalg_norm4(x): from numpy.linalg import norm ; from numpy import inf ; return norm(x, 0)", numpy.arange(6.), linalg_norm4=[NDArray[float,:]])

    def test_linalg_norm5(self):
        self.run_test("def linalg_norm5(x): from numpy.linalg import norm ; from numpy import inf ; return norm(x, ord=inf, axis=1)", (numpy.arange(9) - 4).reshape((3,3)), linalg_norm5=[NDArray[int,:,:]])

    def test_linalg_norm6(self):
        self.run_test("def linalg_norm6(x): from numpy.linalg import norm ; from numpy import inf ; return norm(x, ord=5, axis=(0,))", (numpy.arange(9) - 4).reshape((3,3)), linalg_norm6=[NDArray[int,:,:]])

    def test_linalg_norm7(self):
        self.run_test("def linalg_norm7(x): from numpy.linalg import norm ; return norm(x)", numpy.arange(6).reshape(2,3), linalg_norm7=[NDArray[int,:,:]])

    def test_linalg_norm8(self):
        self.run_test("def linalg_norm8(x): from numpy.linalg import norm ; return norm(x)",
                      numpy.arange(6).reshape(2,3) * 1j + 1,
                      linalg_norm8=[NDArray[complex,:,:]])

    def test_linalg_norm_pydoc(self):
        self.run_test('''
            def linalg_norm_pydoc(x):
                      import numpy as np
                      from numpy import linalg as LA
                      a = np.arange(9) - x
                      b = a.reshape((3, 3))
                      c = np.array([[ 1, 2, 3], [-1, 1, x]])
                      return (LA.norm(a), LA.norm(b),
                              LA.norm(a, np.Inf), #LA.norm(b, np.inf),
                              LA.norm(a, -np.inf), #LA.norm(b, -np.inf),
                              LA.norm(a, 1), #LA.norm(b, 1),
                              LA.norm(a, -1), #LA.norm(b, -1),
                              LA.norm(a, 2), #LA.norm(b, 2),
                              LA.norm(a, -2), #LA.norm(b, -2),
                              LA.norm(a, 3), LA.norm(a, -3),
                              LA.norm(c, axis=0),
                              LA.norm(c, axis=1),
                              LA.norm(c, ord=1, axis=1),
                     )''',
                      10, linalg_norm_pydoc=[int])



#pythran export frequent_itemsets(str list list)
#runas frequent_itemsets([["er", "re", "er"], ["er", "rre", "eeer"], ["e"], ["er"]*8])
import itertools
def frequent_itemsets(sentences):
    # Counts sets with Apriori algorithm.
    SUPP_THRESHOLD = 100
    supps = []

    supp = {}
    for sentence in sentences:
        for key in sentence:
            if key in supp:
                supp[key] += 1
            else:
                supp[key] = 1
    print("|C1| = " + str(len(supp)))
    supps.append({k:v for k,v in supp.items() if v >= SUPP_THRESHOLD})
    print("|L1| = " + str(len(supps[0])))

    supp = {}
    for sentence in sentences:
        for combination in itertools.combinations(sentence, 2):
            if combination[0] in supps[0] and combination[1] in supps[0]:
                key = ','.join(combination)
                if key in supp:
                    supp[key] += 1
                else:
                    supp[key] = 1
    print("|C2| = " + str(len(supp)))
    supps.append({k:v for k,v in supp.items() if v >= SUPP_THRESHOLD})
    print("|L2| = " + str(len(supps[1])))

    supp = {}
    for sentence in sentences:
        for combination in itertools.combinations(sentence, 3):
            if (combination[0]+','+combination[1] in supps[1] and
                    combination[0]+','+combination[2] in supps[1] and
                    combination[1]+','+combination[2] in supps[1]):
                key = ','.join(combination)
                if key in supp:
                    supp[key] += 1
                else:
                    supp[key] = 1
    print("|C3| = " + str(len(supp)))
    supps.append({k:v for k,v in supp.items() if v >= SUPP_THRESHOLD})
    print("|L3| = " + str(len(supps[2])))

    return supps



""" Base file for all Pythran tests. """

import importlib
from numpy import float32, float64, complex128
try:
    from numpy import float128, complex256
except ImportError:
    float128 = float64
    complex256 = complex128

from numpy import intp, intc, number, bool_
from numpy import ndarray, isnan, isinf, isneginf, complex128, complex64
from textwrap import dedent
from threading import Thread
import copy
import math
import glob
import numpy.testing as npt
import os
import numbers
import sys
import unittest
from functools import reduce
from operator import add
import logging

import pytest

from pythran import compile_pythrancode, spec_parser, load_specfile, frontend
from pythran.backend import Python
from pythran.middlend import refine
from pythran.passmanager import PassManager
from pythran.toolchain import _parse_optimization
from pythran.spec import Spec

logger = logging.getLogger("pythran")
logger.setLevel(logging.INFO)

def harmonize_containers(value):
    if isinstance(value, list):
        def flatten(l, flat):
            for e in l:
                if isinstance(e, list):
                    flatten(e, flat)
                else:
                    flat.append(e)
        flat = []
        flatten(value, flat)

        if not flat:
            return

        if any(not isinstance(v, numbers.Number) for v in flat):
            return

        common_type = type(reduce(add, flat))

        def rec_visit_values(l):
            for i, e in enumerate(l):
                if isinstance(e, list):
                    rec_visit_values(e)
                else:
                    l[i] = common_type(e)

        rec_visit_values(value)


class TestEnv(unittest.TestCase):

    """ Test environment to validate a pythran execution against python. """

    module = pytest.mark.module
    # default options used for the c++ compiler
    PYTHRAN_CXX_FLAGS = ['-O0', '-Wall', '-Werror', '-UNDEBUG',
                         '-Wno-unused-function',
                         '-Wno-int-in-bool-context',
                         '-Wno-unknown-warning-option',
                         '-Wno-unused-local-typedefs',
                         '-Wno-absolute-value',
                         '-Wno-missing-braces',
                         '-Wno-unknown-pragmas',
                        ] if sys.platform != "win32" else []

    TEST_RETURNVAL = "TEST_RETURNVAL"

    def check_type(self, ref, res):
        """ Check if type between reference and result match. """
        print("Type of Pythran res : ", type(res))
        print("Type of Python ref : ", type(ref))

        # cope with subtle situation under Windows where numpy.uint32 have same
        # name but different value
        if type(res).__name__ == type(ref).__name__:
            return

        if isinstance(ref, ndarray):
            # res can be an ndarray of dim 0 because of isneginf call
            if ref.ndim == 0 and (not isinstance(res, ndarray)
                                  or res.ndim != 0):
                self.check_type(ref.item(0), res)
            else:
                self.assertIsInstance(res, type(ref))
        elif isinstance(ref, (float, float64)):
            self.assertIsInstance(res, (float, float64))
        elif isinstance(ref, (complex, complex128)):
            self.assertIsInstance(res, (complex, complex128))
        elif isinstance(ref, (bool, bool_)):
            self.assertIsInstance(res, (bool, bool_))
        elif hasattr(ref, 'dtype'):
            if hasattr(res, 'dtype'):
                self.assertEqual(ref.dtype.itemsize, res.dtype.itemsize)
                self.assertEqual(ref.dtype.type(-1), res.dtype.type(-1))
            else:
                self.assertIsInstance(res, int)
        else:
            self.assertIsInstance(res, type(ref))

    def assertAlmostEqual(self, ref, res):
        """
        Improved version of assertAlmostEqual.

        This new version supports nan, complex and ndarray.
        """
        self.check_type(ref, res)
        if isinstance(res, (list, tuple)):
            self.assertEqual(len(res), len(ref))
            for res_v, ref_v in zip(res, ref):
                self.assertAlmostEqual(ref_v, res_v)
        elif isinstance(ref, ndarray):
            npt.assert_array_almost_equal(ref, res)
        elif isinstance(ref, float):
            if isinf(ref) or isinf(res):
                self.assertEqual(isinf(ref), isinf(res))
                self.assertEqual(isneginf(ref), isneginf(res))
            elif isnan(ref) or isnan(res):
                self.assertEqual(isnan(ref), isnan(res))
            elif ref == res:
                pass
            else:
                # Check float equality using upl :
                # http://numscale.github.io/nstest/design_rationale.html
                (m_ref, e_ref) = math.frexp(ref)
                (m_res, e_res) = math.frexp(res)
                expo = max(e_ref, e_res)
                n_ref = math.ldexp(ref, -expo)
                n_res = math.ldexp(res, -expo)
                e = (m_ref - m_res) if e_ref == e_res else (n_ref - n_res)
                self.assertLessEqual(abs(e) / sys.float_info.epsilon, 3.)
        elif isinstance(ref, (complex, complex64, complex128)):
            self.assertAlmostEqual(ref.real, res.real)
            self.assertAlmostEqual(ref.imag, res.imag)
        else:
            self.assertEqual(ref, res)

    def run_python(self, code, runas, prelude=None, check_exception=False):
        """
        Run test with Python to have a reference to compare.

        runas may be a string to run or a
        tuple : (function name, list of parameters).
        """
        # Caller may requires some cleaning
        prelude and prelude()

        # Produce the reference, python-way, run in an separated 'env'
        env = {"builtins": __import__("builtins")}

        # Compare if exception raised in python and in pythran are the same
        err_msg = "Excepected exception but none raise."
        try:
            if isinstance(runas, tuple):
                exec(code, env)
                ret_val = env[runas[0]](*runas[1])
            else:
                exec((code + "\n" + runas), env)
                ret_val = env[self.TEST_RETURNVAL]
            if check_exception:
                raise AssertionError(err_msg)
            return ret_val
        except BaseException as e:
            if not check_exception or (e.args and e.args[0] == err_msg):
                raise
            return type(e)

    def run_pythran(self, modname, module_path, runas, prelude=None,
                    check_exception=False):
        """
        Run Pythran code and clean Pythran dynamic library.

        runas may be a string to run or a
        tuple : (function name, list of parameters).
        """
        # Caller may requires some cleaning
        prelude and prelude()
        loader = importlib.machinery.ExtensionFileLoader(modname, module_path)
        spec = importlib.machinery.ModuleSpec(name=modname, loader=loader,
                                              origin=module_path)
        pymod = importlib._bootstrap._load(spec)

        err_msg = "Excepected exception but none raise."
        try:
            if isinstance(runas, tuple):
                ret_val = getattr(pymod, runas[0])(*runas[1])
            else:
                # Produce the pythran result, exec in the loaded module ctx
                exec(runas, pymod.__dict__)
                ret_val = getattr(pymod, self.TEST_RETURNVAL)
            if check_exception:
                raise AssertionError(err_msg)
            return ret_val
        except BaseException as e:
            if not check_exception or e.args[0] == err_msg:
                raise
            return type(e)

    def cleanup_pythran(self, module_path):
        # Clean temporary DLL
        # FIXME: We can't remove this file while it is used in an import
        # through the exec statement (Windows constraints...)
        if sys.platform != "win32":
            os.remove(module_path)

    def run_test_case(self, code, module_name, runas, module_dir=None, **interface):
        """
        Test if a function call return value is equal for Pythran and Pythran.

        Args:
           code (str):  python (pythran valid) module to test.
           module_name (str): name of the compiled module
           runas (str): command line to run to check output
           interface (dict): pythran interface for the module to test.
                             Each key is the name of a function to call,
                             the value is a list of the arguments' type.

        Returns: nothing.

        Raises:
           AssertionError by 'unittest' if return value differ.
           SyntaxError if code is not python valid.
           pythran.CompileError if generated code can't be compiled.
           ...possibly others...
        """
        # Extract special keys from interface.
        if runas:
            # runas is a python code string to run the test. By convention
            # the last statement of the sequence is the value to test.
            # We insert ourselves a variable to capture this value:
            # "a=1; b=2; myfun(a+b,a-b)" => "a=1; b=2; RES=myfun(a+b,a-b)"
            runas_commands = runas.split(";")
            begin = ";".join(runas_commands[:-1])
            # this tests the runas initialisation syntax
            exec(code + "\n" + begin, {})
            last = self.TEST_RETURNVAL + '=' + runas_commands[-1]
            runas = begin + "\n" + last

        # We run test for each exported function (not for each possible
        # signature.
        for i, name in enumerate(sorted(interface.keys())):
            # If no module name was provided, create one
            modname = (module_name or ("test_" + name)) + str(i)

            # Compile the code using pythran
            cxx_compiled = compile_pythrancode(
                modname, code, interface, module_dir=module_dir, extra_compile_args=self.PYTHRAN_CXX_FLAGS)

            if not runas:
                continue

            python_ref = self.run_python(code, runas)
            pythran_res = self.run_pythran(modname, cxx_compiled, runas)
            self.cleanup_pythran(cxx_compiled)

            print("Python result: ", python_ref)
            print("Pythran result: ", pythran_res)
            harmonize_containers(python_ref)
            self.assertAlmostEqual(python_ref, pythran_res)

    def run_test(self, code, *params, **interface):
        """
        Test if a function call return value is equal for Pythran and Pythran.

        Args:
           code (str):  python (pythran valid) module to test.
           params (tuple): arguments to pass to the function to test.
           prelude (fct): function to call between 'code' and the c++
                          generated code
           interface (dict): pythran interface for the module to test.
                             Each key is the name of a function to call,
                             the value is a list of the arguments' type.
                             Special keys are 'prelude' and 'check_exception'.

        Returns: nothing.

        Raises:
           AssertionError by 'unittest' if return value differ.
           SyntaxError if code is not python valid.
           pythran.CompileError if generated code can't be compiled.
           ...possibly others...
        """
        # Extract special keys from interface.
        prelude = interface.pop('prelude', None)
        check_exception = interface.pop('check_exception', False)
        thread_count = interface.pop('thread_count', 1)

        assert len(interface) == 1

        name = next(iter(interface.keys()))
        modname = "test_" + name

        code = dedent(code)

        cxx_compiled = compile_pythrancode(
            modname, code, interface, extra_compile_args=self.PYTHRAN_CXX_FLAGS)

        # FIXME Check should be done on input parameters after function call
        python_ref = self.run_python(code, (name, copy.deepcopy(params)),
                                     prelude, check_exception)

        run_pythran_args = (modname, cxx_compiled, (name, params),
                            prelude, check_exception)
        pythran_res = self.run_pythran(*run_pythran_args)

        if check_exception:
            if not issubclass(python_ref, pythran_res):
                raise AssertionError(
                    "expected exception was %s, but received %s" %
                    (python_ref, pythran_res))
            else:
                return

        print("Python result: ", python_ref)
        print("Pythran result: ", pythran_res)
        harmonize_containers(python_ref)
        self.assertAlmostEqual(python_ref, pythran_res)

        if thread_count > 1:
            threads = [Thread(target=self.run_pythran,
                              args=run_pythran_args)
                       for _ in range(1, thread_count)]
            for thread in threads:
                thread.start()
            for thread in threads:
                thread.join()
        self.cleanup_pythran(cxx_compiled)


    @staticmethod
    def check_ast(code, ref, optimizations):
        """
        Check if a final node is the same as expected.

        Parameters
        ----------
        code : str
            code we want to check after refine and optimizations
        ref : str
            The expected dump for the AST
        optimizations : [optimization]
            list of optimisation to apply

        Raises
        ------
        is_same : AssertionError
            Raise if the result is not the one expected.
        """
        pm = PassManager("testing")

        ir, _ = frontend.parse(pm, dedent(code))

        optimizations = [_parse_optimization(opt) for opt in optimizations]
        refine(pm, ir, optimizations)

        content = pm.dump(Python, ir)

        if content != dedent(ref).strip():
            raise AssertionError(
                "AST is not the one expected. Reference was %s,"
                "but received %s" % (repr(dedent(ref).strip()), repr(content)))


class TestFromDir(TestEnv):

    """
    Enable tests for a full test cases directory.

    This class load test from individual .py in a directory and expose
    them to the unittest framework. Methods are added to the class (not the
    instance object) because py.test will collect tests by introspection before
    eventually instantiating the class for each test.

    It is intended to be subclassed and then initialized using the static
    populate() method.

    A few class attributes defined the behavior:

    check_output -- Trigger code execution and match return value for Pythran
                    compiled code against pure python. If set to False, only
                    the compilation step is checked.
    files        -- list of files to load, if empty path is used (see below)
    path         -- path where every .py will be loaded
    interface    -- method returning the Pythran interface to use (dict)

    """

    check_output = True
    files = None
    path = "defined_by_subclass"
    runas_markers = ('#runas ', '# runas')

    @staticmethod
    def interface(name=None, file_=None):
        """ Return Pythran specs."""

        # Look for an extra spec file
        spec_file = os.path.splitext(file_)[0] + '.pythran'
        if os.path.isfile(spec_file):
            return load_specfile(spec_file)
        elif file_ is None:
            return Spec({name: []})
        else:
            with open(file_) as fd:
                return spec_parser(fd.read())

    @staticmethod
    def extract_runas(name, filepath):
        with open(filepath) as runas_fd:
            runas_list = [line for line in runas_fd.readlines()
                          if any(line.startswith(marker) for
                                 marker in TestFromDir.runas_markers)]
            return runas_list or [None]

    def __init__(self, *args, **kwargs):
        """ Dynamically add methods for unittests, second stage. """
        TestFromDir.populate(self, stub=False)

        super(TestFromDir, self).__init__(*args, **kwargs)

    class TestFunctor(object):

        """
        Class to holds test information for a given test file.

        This Functor holds for test_* dynamically added method, one per
        input file. It takes at initialization all the informations required
        for a straightforward dispatch to TestEnv.run_test()
        """

        def __init__(self, test_env, module_name, module_code, module_dir,
                     runas=None, **specs):
            self.test_env = test_env
            self.module_name = module_name
            self.module_code = module_code
            self.module_dir = module_dir
            self.runas = runas
            self.specs = specs

        def __name__(self):
            return self.module_name

        def __call__(self):
            if "unittest.skip" in self.module_code:
                return self.test_env.skipTest("Marked as skippable")

            if "unittest.python3.skip" in self.module_code:
                return self.test_env.skipTest("Marked as skippable")

            # resolve import locally to where the tests are located
            sys.path.insert(0, self.test_env.path)

            self.test_env.run_test_case(self.module_code, self.module_name,
                                        self.runas, module_dir=self.module_dir,
                                        **self.specs)
            # restore import path
            sys.path.pop(0)

    @staticmethod
    def populate(target, stub=True):
        """
        Add unittests methods to `target`.

        The python unittest framework detect method named test_* by
        introspection on the class before instantiation. Unfortunately to
        access the TestEnv instance from the method the Functor has to be
        initialized after `target` instantiation. Thus there is a two-stage
        initialization: first we populate the class with 'stub' functions, just
        to satisfy python unittest collect, and then at instantiation the stub
        are replace with the Functor properly initialized with a reference to
        "self".
        """
        if not target.files:
            # No explicit list of files, default to load the whole directory
            target.files = glob.glob(os.path.join(target.path, "*.py"))

        for filepath in target.files:
            # Module name is file name and external interface is default value
            name, _ = os.path.splitext(os.path.basename(filepath))
            specs = target.interface(name, filepath).functions

            runas_list = target.extract_runas(name, filepath)
            for n, runas in enumerate(runas_list):
                if runas:
                    # Remove the runas marker
                    runas = runas[runas.find(' ', 2) + 1:]
                    suffix = "_run"
                else:
                    suffix = '_norun'

                if stub:
                    # First stage, we add dummy function.
                    def func():
                        """ Useless function to populate TestCase. """
                        return
                else:
                    # Second stage, we change dummy function by real one.
                    with open(filepath) as fd:
                        func = TestFromDir.TestFunctor(
                            target, name + suffix + str(n), fd.read(),
                            os.path.dirname(filepath),
                            runas=runas, **specs)
                setattr(target, "test_" + name + suffix + str(n), func)



# coding: utf-8
import numpy as np
from math import log, tanh

#pythran export phi0(float)
def phi0(x):
    x = abs(x)
    if (x < 9.08e-5 ):
        return( 10 );
    else:
        return -log (tanh (x/2))


#pythran export G(float[:])
def G(Lq):
    X = sum (phi0(e) for e in Lq)
    s = np.prod(np.sign(Lq))
    return s * phi0(X)

#pythran export BinaryProduct(int[:,:], int[:])
def BinaryProduct(X,Y):
        
    """ Binary Matrices or Matrix-vector product in Z/2Z. Works with scipy.sparse.csr_matrix matrices X,Y too."""
 
    A = X.dot(Y)
    
    return A%2

#pythran export InCode(int[:,:], int[:])
def InCode(H,x):

    """ Computes Binary Product of H and x. If product is null, x is in the code.
        Returns appartenance boolean. 
    """
        
    return  (BinaryProduct(H,x)==0).all()

#pythran export Decoding_logBP(int[:,:], int list list, int list list, float[:,:], float[:], int)
def Decoding_logBP(H,Bits,Nodes,Lq,Lc,max_iter=1):
    """ Decoding function using Belief Propagation algorithm (logarithmic version)
    IMPORTANT: if H is large (n>1000), H should be scipy.sparse.csr_matrix object to speed up calculations
    (highly recommanded. )
    -----------------------------------
    Parameters:
    
    H: 2D-array (OR scipy.sparse.csr_matrix object) Parity check matrix, shape = (m,n) 
    y: n-vector recieved after transmission in the channel. (In general, returned 
    by Coding Function)
    Signal-Noise Ratio: SNR = 10log(1/variance) in decibels of the AWGN used in coding.
    
    max_iter: (default = 1) max iterations of the main loop. Increase if decoding is not error-free.
     """
        
    m,n=H.shape

    if not len(Lc)==n:
        raise ValueError('La taille de y doit correspondre au nombre de colonnes de H')

    if m>=n:
        raise ValueError('H doit avoir plus de colonnes que de lignes')
    
    
    # var = 10**(-SNR/10)

    # ### ETAPE 0: initialisation 
    
    # Lc = 2*y/var

    # Lq=np.zeros(shape=(m,n))

    Lr = np.zeros(shape=(m,n))
    
    count=0
    
    # prod=np.prod
    # tanh = np.tanh
    # log = np.log
    
    Lq += Lc
    
    #Bits,Nodes = BitsAndNodes(H)
    while(True):

        count+=1 #Compteur qui empêche la boucle d'être infinie .. 

        #### ETAPE 1 : Horizontale
        for i in range(m):
            Ni = Bits[i]
            for j in Ni:
                Nij = list(Ni)
                Nij.remove(j)
                # X = sum (phi0(e) for e in Lq[i,Nij])
                # s = prod(np.sign(Lq[i,Nij]))
                # Lr[i,j] = s * phi0(X)
                Lr[i,j] = G(Lq[i][Nij])
        Lr = np.clip (Lr, -100, 100)

        #### ETAPE 2 : Verticale
        for j in range(n):
            Mj = Nodes[j]
            
            for i in Mj:
                Mji = list(Mj)
                Mji.remove(i)

                Lq[i,j] = Lc[j]+sum(Lr[Mji][:,j])
        
 
        #### LLR a posteriori:
        # L_posteriori = np.zeros(n)
        # for j in range(n):
        #     Mj = Nodes[j]

        #     L_posteriori[j] = Lc[j] + sum(Lr[Mj,j])
        extrinsic = np.empty(n)
        for j in range(n):
            Mj = Nodes[j]

            extrinsic[j] = sum(Lr[Mj][:,j])

        L_posteriori = extrinsic + Lc
        #x = np.array(L_posteriori <= 0).astype(int)
        x = np.array(extrinsic <= 0).astype(int)
        product = InCode(H,x)
        #print(count, product)

        if product or count >= max_iter:
            break
    # print(count)
    return np.array(L_posteriori <= 0).astype(int), Lq - Lc, extrinsic, product



from pythran.tests import TestEnv
from pythran.syntax import PythranSyntaxError

class TestNamedParameters(TestEnv):

    def test_call_with_named_argument(self):
        self.run_test("""
def foo(a):
    return a
def call_with_named_argument(n):
    return foo(a=n)""", 1, call_with_named_argument=[int])

    def test_call_with_named_arguments(self):
        self.run_test("""
def foo(a,b):
    return a / b
def call_with_named_arguments(n):
    return foo(b=n, a=2*n)""", 1, call_with_named_arguments=[int])

    def test_call_with_args_and_named_argument(self):
        self.run_test("""
def foo(a, b):
    return a - b
def call_with_args_and_named_argument(m,n):
    return foo(m, b=n)""", 1, 2, call_with_args_and_named_argument=[int, int])

    def test_call_with_args_and_named_arguments(self):
        self.run_test("""
def foo(a,b,c):
    return c + a / b
def call_with_args_and_named_arguments(n, m):
    return foo(m, c=2*n, b=n)""", 1, 2, call_with_args_and_named_arguments=[int, int])

    def test_call_with_default_and_named_argument(self):
        self.run_test("""
def foo(a, b=1):
    return a - b
def call_with_default_and_named_argument(m,n):
    return foo(a=m)""", 1, 2, call_with_default_and_named_argument=[int, int])

    def test_call_with_default_and_named_arguments(self):
        self.run_test("""
def foo(a,b,c=1):
    return c + a / b
def call_with_default_and_named_arguments(n, m):
    return foo(m, b=n)""", 1, 2, call_with_default_and_named_arguments=[int, int])

    def test_intrinsic_named_argument(self):
        """ Check named arguments with attributes as value. """
        self.run_test("""
            def intrinsic_named_argument(n):
                import numpy
                return numpy.ones(n, dtype=numpy.uint8).nbytes""",
                      4, intrinsic_named_argument=[int])

    def test_intrinsic_named_argument_without_default(self):
        self.run_test("""
            def intrinsic_named_argument_without_default(n):
                import numpy as np
                return np.expand_dims(np.ones(n), axis=0)""",
                      4, intrinsic_named_argument_without_default=[int])

    def test_nested_function_with_named_arguments(self):
        self.run_test('''
def nested_function_with_named_arguments(a):
    b = a * 2
    def foo(c):
        return b + c
    return foo(c=a)''',
        4, nested_function_with_named_arguments=[int])

    def test_nested_function_with_several_named_arguments(self):
        self.run_test('''
def nested_function_with_several_named_arguments(a):
    b = a * 2
    def foo(c, e):
        return b + c + e
    return foo(e = 4, c=a)''',
        4, nested_function_with_several_named_arguments=[int])

    def test_aliasing_functions_with_named_arguments(self):
        self.run_test('''
            def aliasing_functions_with_named_arguments(n):
                import numpy
                if n > 10:
                    my = numpy.ones
                else:
                    my = numpy.zeros
                return my(n, dtype=numpy.uint8).nbytes''',
                      4, aliasing_functions_with_named_arguments=[int])

    def test_aliasing_functions_with_different_structural_types(self):
        with self.assertRaises(PythranSyntaxError):
            self.run_test('''
def aliasing_functions_with_different_structural_types(n):
    import numpy
    if n > 10:
        my = sum
    else:
        my = numpy.zeros
    return my(n, dtype=numpy.uint8).nbytes''',
            4, aliasing_functions_with_different_structural_types=[int])

    def test_default_argument_all_filled(self):
        code = '''
            def default_argument_all_filled(x):
                return test2(x,2)

            def test2(a, b=3):
                return a, b'''
        self.run_test(code, 10, default_argument_all_filled=[int])




#from http://wiki.scipy.org/Cookbook/Theoretical_Ecology/Hastings_and_Powell
#pythran export fweb(float [], float, float, float, float, float, float, float)
import numpy as np
def fweb(y, t, a1, a2, b1, b2, d1, d2):
    yprime = np.empty((3,))
    yprime[0] = y[0] * (1. - y[0]) - a1*y[0]*y[1]/(1. + b1 * y[0])
    yprime[1] = a1*y[0]*y[1] / (1. + b1 * y[0]) - a2 * y[1]*y[2] / (1. + b2 * y[1]) - d1 * y[1]
    yprime[2] = a2*y[1]*y[2]/(1. + b2*y[1]) - d2*y[2]
    return yprime



#from: http://stackoverflow.com/questions/16541618/perform-a-reverse-cumulative-sum-on-a-numpy-array
#pythran export reverse_cumsum(float[])
#runas import numpy as np ; r = np.arange(10, dtype=float) ; reverse_cumsum(r)
#bench import numpy as np ; r = np.arange(1000000, dtype=float) ; reverse_cumsum(r)
import numpy as np
def reverse_cumsum(x):
    return np.cumsum(x[::-1])[::-1]



def omp_parallel_for_if(using=0):
    num_threads = 0
    import omp
    sum = 0
    sum2 = 0
    LOOPCOUNT=1000
    'omp parallel for if(using == 1)'
    for i in range(LOOPCOUNT+1):
        num_threads = omp.get_num_threads()
        sum+=i
    known_sum = (LOOPCOUNT * (LOOPCOUNT + 1)) / 2
    return known_sum == sum and num_threads == 1



#pythran export solve(int)
#runas solve(10001)
def solve(p):
    '''
    By listing the first six prime numbers: 2, 3, 5, 7, 11, and 13, we can see that the 6th prime is 13.

    What is the 10001st prime number?
    '''

    prime_list = [2, 3, 5, 7, 11, 13, 17, 19, 23]   # Ensure that this is initialised with at least 1 prime
    prime_dict = dict.fromkeys(prime_list, 1)

    def _isprime(n):
        ''' Raw check to see if n is prime. Assumes that prime_list is already populated '''
        isprime = n >= 2 and 1 or 0
        for prime in prime_list:                    # Check for factors with all primes
            if prime * prime > n: break             # ... up to sqrt(n)
            if not n % prime:
                isprime = 0
                break
        if isprime: prime_dict[n] = 1               # Maintain a dictionary for fast lookup
        return isprime

    def prime(x):
        ''' Returns the xth prime '''

        lastn = prime_list[-1]
        while len(prime_list) <= x:                 # Keep working until we've got the xth prime
            lastn = lastn + 1                       # Check the next number
            if _isprime(lastn):
                prime_list.append(lastn)            # Maintain a list for sequential access
        return prime_list[x]

    return prime(p - 1)



#pythran export _Aij(float[:,:], int, int)
#pythran export _Aij(int[:,:], int, int)
def _Aij(A, i, j):
    """Sum of upper-left and lower right blocks of contingency table."""
    # See `somersd` References [2] bottom of page 309
    return A[:i, :j].sum() + A[i+1:, j+1:].sum()

#pythran export _Dij(float[:,:], int, int)
#pythran export _Dij(int[:,:], int, int)
def _Dij(A, i, j):
    """Sum of lower-left and upper-right blocks of contingency table."""
    # See `somersd` References [2] bottom of page 309
    return A[i+1:, :j].sum() + A[:i, j+1:].sum()

#pythran export _P(float[:,:])
#pythran export _P(int[:,:])
def _P(A):
    """Twice the number of concordant pairs, excluding ties."""
    # See `somersd` References [2] bottom of page 309
    m, n = A.shape
    count = 0
    for i in range(m):
        for j in range(n):
            count += A[i, j]*_Aij(A, i, j)
    return count

#pythran export _Q(float[:,:])
#pythran export _Q(int[:,:])
def _Q(A):
    """Twice the number of discordant pairs, excluding ties."""
    # See `somersd` References [2] bottom of page 309
    m, n = A.shape
    count = 0
    for i in range(m):
        for j in range(n):
            count += A[i, j]*_Dij(A, i, j)
    return count

#pythran export _a_ij_Aij_Dij2(float[:,:])
#pythran export _a_ij_Aij_Dij2(int[:,:])
def _a_ij_Aij_Dij2(A):
    """A term that appears in the ASE of Kendall's tau and Somers' D."""
    # See `somersd` References [2] section 4: Modified ASEs to test the null hypothesis...
    m, n = A.shape
    count = 0
    for i in range(m):
        for j in range(n):
            count += A[i, j]*(_Aij(A, i, j) - _Dij(A, i, j))**2
    return count


#skip.runas import Image; im = Image.open("Scribus.gif"); image_list = list(im.getdata()); cols, rows = im.size; res = range(len(image_list)); sobelFilter(image_list, res, cols, rows)
#runas cols = 100; rows = 100 ;image_list=[x%10+y%20 for x in range(cols) for y in range(rows)]; sobelFilter(image_list, cols, rows)
#bench cols = 1000; rows = 500 ;image_list=[x%10+y%20 for x in range(cols) for y in range(rows)]; sobelFilter(image_list, cols, rows)
#pythran export sobelFilter(int list, int, int)
def sobelFilter(original_image, cols, rows):
    edge_image = list(range(len(original_image)))
    for i in range(rows):
        edge_image[i * cols] = 255
        edge_image[((i + 1) * cols) - 1] = 255

    for i in range(1, cols - 1):
        edge_image[i] = 255
        edge_image[i + ((rows - 1) * cols)] = 255

    for iy in range(1, rows - 1):
        for ix in range(1, cols - 1):
            sum_x = 0
            sum_y = 0
            sum = 0
            #x gradient approximation
            sum_x += original_image[ix - 1 + (iy - 1) * cols] * -1
            sum_x += original_image[ix + (iy - 1) * cols] * -2
            sum_x += original_image[ix + 1 + (iy - 1) * cols] * -1
            sum_x += original_image[ix - 1 + (iy + 1) * cols] * 1
            sum_x += original_image[ix + (iy + 1) * cols] * 2
            sum_x += original_image[ix + 1 + (iy + 1) * cols] * 1
            sum_x = min(255, max(0, sum_x))
            #y gradient approximatio
            sum_y += original_image[ix - 1 + (iy - 1) * cols] * 1
            sum_y += original_image[ix + 1 + (iy - 1) * cols] * -1
            sum_y += original_image[ix - 1 + (iy) * cols] * 2
            sum_y += original_image[ix + 1 + (iy) * cols] * -2
            sum_y += original_image[ix - 1 + (iy + 1) * cols] * 1
            sum_y += original_image[ix + 1 + (iy + 1) * cols] * -1
            sum_y = min(255, max(0, sum_y))

            #GRADIENT MAGNITUDE APPROXIMATION
            sum = abs(sum_x) + abs(sum_y)

            #make edges black and background white
            edge_image[ix + iy * cols] = 255 - (255 & sum)
    return edge_image



# cython: np_pythran=True

import numpy as np
cimport numpy as np

def indexing():
    cdef np.ndarray[double, ndim=2] A = np.random.rand(1, 1)
    cdef double a
    with nogil:
        A[0, 0] = 12.
        a = A[0, 0]
    return a




import numpy as np

def perm(data, p):
    n = len(data)
    out = np.empty(n)
    #pragma omp simd
    for i in range(n):
        out[i] = data[p[i]] * 2
    return out

def simd():
    data = [10, 20, 30, 40]
    p = [1, 0, 3, 2]
    res = perm(data, p)
    return np.all(res == [40, 20, 80, 60])



import unittest
from pythran.tests import TestEnv
import numpy
from pythran.typing import NDArray
import unittest


@TestEnv.module
class TestNumpyRFFT(TestEnv):

    # Basic test
    def test_rfft_0(self):
        self.run_test("def test_rfft(x): from numpy.fft import rfft ; return rfft(x)", numpy.arange(0,8.), test_rfft=[NDArray[float,:]])
    # Test various values of n, even, odd, greater and smaller than array size
    def test_rfft_1(self):
        self.run_test("def test_rfft_1(x,n): from numpy.fft import rfft ; return rfft(x,n)", numpy.arange(0,8.),8, test_rfft_1=[NDArray[float,:],int])
    def test_rfft_2(self):
        self.run_test("def test_rfft_2(x,n): from numpy.fft import rfft ; return rfft(x,n)", numpy.arange(0,8.),9, test_rfft_2=[NDArray[float,:],int])
    def test_rfft_3(self):
        self.run_test("def test_rfft_3(x,n): from numpy.fft import rfft ; return rfft(x,n)", numpy.arange(0,8.),7, test_rfft_3=[NDArray[float,:],int])
    def test_rfft_4(self):
        self.run_test("def test_rfft_4(x,n): from numpy.fft import rfft ; return rfft(x,n)", numpy.arange(0,8.),6, test_rfft_4=[NDArray[float,:],int])
    def test_rfft_5(self):
        self.run_test("def test_rfft_5(x,n): from numpy.fft import rfft ; return rfft(x,n)", numpy.arange(0,8.),10, test_rfft_5=[NDArray[float,:],int])

    # Two dimensional array
    def test_rfft_6(self):
        self.run_test("def test_rfft_6(x): from numpy.fft import rfft ; return rfft(x)", numpy.random.random((4,128)), test_rfft_6=[NDArray[float,:,:]])
    # Test axes
    def test_rfft_7(self):
        self.run_test("def test_rfft_7(x,n,a): from numpy.fft import rfft ; return rfft(x,n,a)", numpy.random.random((4,128)),128,1, test_rfft_7=[NDArray[float,:,:],int,int])
    def test_rfft_8(self):
        self.run_test("def test_rfft_8(x,n,a): from numpy.fft import rfft ; return rfft(x,n,a)", numpy.random.random((4,128)),128,0, test_rfft_8=[NDArray[float,:,:],int,int])
    # Test renorm
    def test_rfft_9(self):
        self.run_test("def test_rfft_9(x,n,a,r): from numpy.fft import rfft ; return rfft(x,n,a,r)", numpy.random.random((4,128)),128,1,'ortho', test_rfft_9=[NDArray[float,:,:],int,int,str])

    # Test float32
    def test_rfft_10(self):
        self.run_test("def test_rfft_10(x): from numpy.fft import rfft ; return rfft(x)", numpy.arange(0,8.).astype(numpy.float32), test_rfft_10=[NDArray[numpy.float32,:]])
    def test_rfft_11(self):
        self.run_test("def test_rfft_11(x,n): from numpy.fft import rfft ; return rfft(x,n)", numpy.arange(0,8.).astype(numpy.float32),16,test_rfft_11=[NDArray[numpy.float32,:],int])

    # Test parallel:
    def test_rfft_12(self):
        self.run_test('''
import numpy as np
def test_rfft_12(x):
    out = out = [np.empty_like(x, dtype=complex) for i in range(20)] 
    #omp parallel for
    for ii in range(20):
        out[ii] = np.fft.rfft(x)
    return np.concatenate(out)
''',numpy.random.random((4,128)), test_rfft_12=[NDArray[float,:,:]])

    # Test with arguments
    def test_rfft_13(self):
        self.run_test("def test_rfft_13(x): from numpy.fft import rfft ; return rfft(x,axis=1)", numpy.random.random((2,128)), test_rfft_13=[NDArray[float,:,:]])

    def test_rfft_14(self):
        self.run_test("def test_rfft_14(x): from numpy.fft import rfft ; return rfft(x,n=128,axis=0)", numpy.random.random((2,128)), test_rfft_14=[NDArray[float,:,:]])

    @unittest.skip("Mismatch because numpy converts to double before fft (See comment in header file)")
    def test_rfft_f32(self):
        self.run_test("def test_rfft_f32(x): from numpy.fft import rfft ; return rfft(x)", numpy.random.random(128).astype(numpy.float32), test_rfft_f32=[NDArray[numpy.float32,:]])

    def test_rfft_int64(self):
        self.run_test("def test_rfft_int64(x): from numpy.fft import rfft ; return rfft(x)", (100*numpy.random.random(128)).astype(numpy.int64), test_rfft_int64=[NDArray[numpy.int64,:]])

    def test_rfft_byte(self):
        self.run_test("def test_rfft_byte(x): from numpy.fft import rfft ; return rfft(x)", (100*numpy.random.random(128)).astype(numpy.byte), test_rfft_byte=[NDArray[numpy.byte,:]])


@TestEnv.module
class TestNumpyIRFFT(TestEnv):
    ############# IRFFT
    # Basic test
    def test_irfft_0(self):
        self.run_test("def test_irfft_0(x): from numpy.fft import irfft ; return irfft(x)", numpy.exp(1j*numpy.random.random(8)), test_irfft_0=[NDArray[complex,:]])
    # Test various values of n, even, odd, greater and smaller than array size
    def test_irfft_1(self):
        self.run_test("def test_irfft_1(x,n): from numpy.fft import irfft ; return irfft(x,n)", numpy.exp(1j*numpy.random.random(8)),8, test_irfft_1=[NDArray[complex,:],int])
    def test_irfft_2(self):
        self.run_test("def test_irfft_2(x,n): from numpy.fft import irfft ; return irfft(x,n)", numpy.exp(1j*numpy.random.random(8)),9, test_irfft_2=[NDArray[complex,:],int])
    def test_irfft_3(self):
        self.run_test("def test_irfft_3(x,n): from numpy.fft import irfft ; return irfft(x,n)", numpy.exp(1j*numpy.random.random(8)),7, test_irfft_3=[NDArray[complex,:],int])
    def test_irfft_4(self):
        self.run_test("def test_irfft_4(x,n): from numpy.fft import irfft ; return irfft(x,n)", numpy.exp(1j*numpy.random.random(8)),6, test_irfft_4=[NDArray[complex,:],int])
    def test_irfft_5(self):
        self.run_test("def test_irfft_5(x,n): from numpy.fft import irfft ; return irfft(x,n)", numpy.exp(1j*numpy.random.random(8)),10, test_irfft_5=[NDArray[complex,:],int])

    # Two dimensional array
    def test_irfft_6(self):
        self.run_test("def test_irfft_6(x): from numpy.fft import irfft ; return irfft(x)", numpy.exp(1j*numpy.random.random((4,128))), test_irfft_6=[NDArray[complex,:,:]])
    # Test axes
    def test_irfft_7(self):
        self.run_test("def test_irfft_7(x,n,a): from numpy.fft import irfft ; return irfft(x,n,a)", numpy.exp(1j*numpy.random.random((4,128))),128,1, test_irfft_7=[NDArray[complex,:,:],int,int])
    def test_irfft_8(self):
        self.run_test("def test_irfft_8(x,n,a): from numpy.fft import irfft ; return irfft(x,n,a)", numpy.exp(1j*numpy.random.random((4,128))),128,0, test_irfft_8=[NDArray[complex,:,:],int,int])
    # Test renorm
    def test_irfft_9(self):
        self.run_test("def test_irfft_9(x,n,a,r): from numpy.fft import irfft ; return irfft(x,n,a,r)", numpy.exp(1j*numpy.random.random((4,128))),128,1,'ortho', test_irfft_9=[NDArray[complex,:,:],int,int,str])

    # Test complex64
    def test_irfft_10(self):
        self.run_test("def test_irfft_10(x): from numpy.fft import irfft ; return irfft(x)", numpy.exp(1j*numpy.random.random(8)).astype(numpy.complex64), test_irfft_10=[NDArray[numpy.complex64,:]])
    def test_irfft_11(self):
        self.run_test("def test_irfft_11(x,n): from numpy.fft import irfft ; return irfft(x,n)", numpy.exp(1j*numpy.random.random(8)).astype(numpy.complex64),16,test_irfft_11=[NDArray[numpy.complex64,:],int])

    # Test parallel:
    def test_irfft_12(self):
        self.run_test('''
import numpy as np
def test_irfft_12(x):
    out = [np.empty_like(x, dtype=float) for i in range(20)] 
    #omp parallel for
    for ii in range(20):
        out[ii] = np.fft.irfft(x)
    return np.concatenate(out)
''',numpy.exp(1j*numpy.random.random((4,128))).astype(numpy.complex64), test_irfft_12=[NDArray[numpy.complex64,:,:]])

    # Test with arguments
    def test_irfft_13(self):
        self.run_test("def test_irfft_13(x): from numpy.fft import irfft ; return irfft(x,axis=1)", numpy.exp(1j*numpy.random.random((2,128))), test_irfft_13=[NDArray[complex,:,:]])

    def test_irfft_14(self):
        self.run_test("def test_irfft_14(x): from numpy.fft import irfft ; return irfft(x,n=128,axis=0)", numpy.exp(1j*numpy.random.random((2,128))), test_irfft_14=[NDArray[complex,:,:]])
        
    def test_irfft_c64(self):
        self.run_test("def test_irfft_c64(x): from numpy.fft import irfft ; return irfft(x)", numpy.exp(1j*numpy.random.random((2,128))).astype(numpy.complex64), test_irfft_c64=[NDArray[numpy.complex64,:,:]])
        
    def test_irfft_f64(self):
        self.run_test("def test_irfft_f64(x): from numpy.fft import irfft ; return irfft(x)", numpy.random.random(128), test_irfft_f64=[NDArray[numpy.float64,:]])
        
    def test_irfft_f32(self):
        self.run_test("def test_irfft_f32(x): from numpy.fft import irfft ; return irfft(x)", numpy.random.random(128).astype(numpy.float32), test_irfft_f32=[NDArray[numpy.float32,:]])
        
    def test_irfft_int64(self):
        self.run_test("def test_irfft_int64(x): from numpy.fft import irfft ; return irfft(x)", (100*numpy.random.random(128)).astype(numpy.int64), test_irfft_int64=[NDArray[numpy.int64,:]])

    def test_irfft_byte(self):
        self.run_test("def test_irfft_byte(x): from numpy.fft import irfft ; return irfft(x)", (100*numpy.random.random(128)).astype(numpy.byte), test_irfft_byte=[NDArray[numpy.byte,:]])

@TestEnv.module
class TestNumpyIHFFT(TestEnv):

    # Basic test
    def test_ihfft_0(self):
        self.run_test("def test_ihfft(x): from numpy.fft import ihfft ; return ihfft(x)", numpy.arange(0,8.), test_ihfft=[NDArray[float,:]])
    # Test various values of n, even, odd, greater and smaller than array size
    def test_ihfft_1(self):
        self.run_test("def test_ihfft_1(x,n): from numpy.fft import ihfft ; return ihfft(x,n)", numpy.arange(0,8.),8, test_ihfft_1=[NDArray[float,:],int])
    def test_ihfft_2(self):
        self.run_test("def test_ihfft_2(x,n): from numpy.fft import ihfft ; return ihfft(x,n)", numpy.arange(0,8.),9, test_ihfft_2=[NDArray[float,:],int])
    def test_ihfft_3(self):
        self.run_test("def test_ihfft_3(x,n): from numpy.fft import ihfft ; return ihfft(x,n)", numpy.arange(0,8.),7, test_ihfft_3=[NDArray[float,:],int])
    def test_ihfft_4(self):
        self.run_test("def test_ihfft_4(x,n): from numpy.fft import ihfft ; return ihfft(x,n)", numpy.arange(0,8.),6, test_ihfft_4=[NDArray[float,:],int])
    def test_ihfft_5(self):
        self.run_test("def test_ihfft_5(x,n): from numpy.fft import ihfft ; return ihfft(x,n)", numpy.arange(0,8.),10, test_ihfft_5=[NDArray[float,:],int])

    # Two dimensional array
    def test_ihfft_6(self):
        self.run_test("def test_ihfft_6(x): from numpy.fft import ihfft ; return ihfft(x)", numpy.random.random((4,128)), test_ihfft_6=[NDArray[float,:,:]])
    # Test axes
    def test_ihfft_7(self):
        self.run_test("def test_ihfft_7(x,n,a): from numpy.fft import ihfft ; return ihfft(x,n,a)", numpy.random.random((4,128)),128,1, test_ihfft_7=[NDArray[float,:,:],int,int])
    def test_ihfft_8(self):
        self.run_test("def test_ihfft_8(x,n,a): from numpy.fft import ihfft ; return ihfft(x,n,a)", numpy.random.random((4,128)),128,0, test_ihfft_8=[NDArray[float,:,:],int,int])
    # Test renorm
    def test_ihfft_9(self):
        self.run_test("def test_ihfft_9(x,n,a,r): from numpy.fft import ihfft ; return ihfft(x,n,a,r)", numpy.random.random((4,128)),128,1,'ortho', test_ihfft_9=[NDArray[float,:,:],int,int,str])

    # Test float32
    def test_ihfft_10(self):
        self.run_test("def test_ihfft_10(x): from numpy.fft import ihfft ; return ihfft(x)", numpy.arange(0,8.).astype(numpy.float32), test_ihfft_10=[NDArray[numpy.float32,:]])
    def test_ihfft_11(self):
        self.run_test("def test_ihfft_11(x,n): from numpy.fft import ihfft ; return ihfft(x,n)", numpy.arange(0,8.).astype(numpy.float32),16,test_ihfft_11=[NDArray[numpy.float32,:],int])

    # Test parallel:
    def test_ihfft_12(self):
        self.run_test('''
import numpy as np
def test_ihfft_12(x):
    out = out = [np.empty_like(x, dtype=complex) for i in range(20)] 
    #omp parallel for
    for ii in range(20):
        out[ii] = np.fft.ihfft(x)
    return np.concatenate(out)
''',numpy.random.random((4,128)), test_ihfft_12=[NDArray[float,:,:]])

    # Test with arguments
    def test_ihfft_13(self):
        self.run_test("def test_ihfft_13(x): from numpy.fft import ihfft ; return ihfft(x,axis=1)", numpy.random.random((2,128)), test_ihfft_13=[NDArray[float,:,:]])

    def test_ihfft_14(self):
        self.run_test("def test_ihfft_14(x): from numpy.fft import ihfft ; return ihfft(x,n=128,axis=0)", numpy.random.random((2,128)), test_ihfft_14=[NDArray[float,:,:]])

    @unittest.skip("Mismatch because numpy converts to double before fft (See comment in header file)")
    def test_ihfft_f32(self):
        self.run_test("def test_ihfft_f32(x): from numpy.fft import ihfft ; return ihfft(x)", numpy.random.random(128).astype(numpy.float32), test_ihfft_f32=[NDArray[numpy.float32,:]])

    def test_ihfft_int64(self):
        self.run_test("def test_ihfft_int64(x): from numpy.fft import ihfft ; return ihfft(x)", (100*numpy.random.random(128)).astype(numpy.int64), test_ihfft_int64=[NDArray[numpy.int64,:]])

    def test_ihfft_byte(self):
        self.run_test("def test_ihfft_byte(x): from numpy.fft import ihfft ; return ihfft(x)", (100*numpy.random.random(128)).astype(numpy.byte), test_ihfft_byte=[NDArray[numpy.byte,:]])


@TestEnv.module
class TestNumpyHFFT(TestEnv):
    ############# hfft
    # Basic test
    def test_hfft_0(self):
        self.run_test("def test_hfft_0(x): from numpy.fft import hfft ; return hfft(x)", numpy.exp(1j*numpy.random.random(8)), test_hfft_0=[NDArray[complex,:]])
    # Test various values of n, even, odd, greater and smaller than array size
    def test_hfft_1(self):
        self.run_test("def test_hfft_1(x,n): from numpy.fft import hfft ; return hfft(x,n)", numpy.exp(1j*numpy.random.random(8)),8, test_hfft_1=[NDArray[complex,:],int])
    def test_hfft_2(self):
        self.run_test("def test_hfft_2(x,n): from numpy.fft import hfft ; return hfft(x,n)", numpy.exp(1j*numpy.random.random(8)),9, test_hfft_2=[NDArray[complex,:],int])
    def test_hfft_3(self):
        self.run_test("def test_hfft_3(x,n): from numpy.fft import hfft ; return hfft(x,n)", numpy.exp(1j*numpy.random.random(8)),7, test_hfft_3=[NDArray[complex,:],int])
    def test_hfft_4(self):
        self.run_test("def test_hfft_4(x,n): from numpy.fft import hfft ; return hfft(x,n)", numpy.exp(1j*numpy.random.random(8)),6, test_hfft_4=[NDArray[complex,:],int])
    def test_hfft_5(self):
        self.run_test("def test_hfft_5(x,n): from numpy.fft import hfft ; return hfft(x,n)", numpy.exp(1j*numpy.random.random(8)),10, test_hfft_5=[NDArray[complex,:],int])

    # Two dimensional array
    def test_hfft_6(self):
        self.run_test("def test_hfft_6(x): from numpy.fft import hfft ; return hfft(x)", numpy.exp(1j*numpy.random.random((4,128))), test_hfft_6=[NDArray[complex,:,:]])
    # Test axes
    def test_hfft_7(self):
        self.run_test("def test_hfft_7(x,n,a): from numpy.fft import hfft ; return hfft(x,n,a)", numpy.exp(1j*numpy.random.random((4,128))),128,1, test_hfft_7=[NDArray[complex,:,:],int,int])
    def test_hfft_8(self):
        self.run_test("def test_hfft_8(x,n,a): from numpy.fft import hfft ; return hfft(x,n,a)", numpy.exp(1j*numpy.random.random((4,128))),128,0, test_hfft_8=[NDArray[complex,:,:],int,int])
    # Test renorm
    def test_hfft_9(self):
        self.run_test("def test_hfft_9(x,n,a,r): from numpy.fft import hfft ; return hfft(x,n,a,r)", numpy.exp(1j*numpy.random.random((4,128))),128,1,'ortho', test_hfft_9=[NDArray[complex,:,:],int,int,str])

    # Test complex64
    def test_hfft_10(self):
        self.run_test("def test_hfft_10(x): from numpy.fft import hfft ; return hfft(x)", numpy.exp(1j*numpy.random.random(8)).astype(numpy.complex64), test_hfft_10=[NDArray[numpy.complex64,:]])
    def test_hfft_11(self):
        self.run_test("def test_hfft_11(x,n): from numpy.fft import hfft ; return hfft(x,n)", numpy.exp(1j*numpy.random.random(8)).astype(numpy.complex64),16,test_hfft_11=[NDArray[numpy.complex64,:],int])

    # Test parallel:
    def test_hfft_12(self):
        self.run_test('''
import numpy as np
def test_hfft_12(x):
    out = [np.empty_like(x, dtype=float) for i in range(20)] 
    #omp parallel for
    for ii in range(20):
        out[ii] = np.fft.hfft(x)
    return np.concatenate(out)
''',numpy.exp(1j*numpy.random.random((4,128))), test_hfft_12=[NDArray[numpy.complex128,:,:]])

    # Test with arguments
    def test_hfft_13(self):
        self.run_test("def test_hfft_13(x): from numpy.fft import hfft ; return hfft(x,axis=1)", numpy.exp(1j*numpy.random.random((2,128))), test_hfft_13=[NDArray[complex,:,:]])

    def test_hfft_14(self):
        self.run_test("def test_hfft_14(x): from numpy.fft import hfft ; return hfft(x,n=128,axis=0)", numpy.exp(1j*numpy.random.random((2,128))), test_hfft_14=[NDArray[complex,:,:]])

    @unittest.skip("Fails because of numpy casting to  double (see documentation in headers for a discussion)")
    def test_hfft_c64(self):
        self.run_test("def test_hfft_c64(x): from numpy.fft import hfft ; return hfft(x)", numpy.exp(1j*numpy.random.random((2,128))).astype(numpy.complex64), test_hfft_c64=[NDArray[numpy.complex64,:,:]])

    def test_hfft_f64(self):
        self.run_test("def test_hfft_f64(x): from numpy.fft import hfft ; return hfft(x)", numpy.random.random(128), test_hfft_f64=[NDArray[numpy.float64,:]])

    @unittest.skip("Fails because of numpy casting to  double (see documentation in headers for a discussion)")
    def test_hfft_f32(self):
        self.run_test("def test_hfft_f32(x): from numpy.fft import hfft ; return hfft(x)", numpy.random.random(128).astype(numpy.float32), test_hfft_f32=[NDArray[numpy.float32,:]])

    def test_hfft_int64(self):
        self.run_test("def test_hfft_int64(x): from numpy.fft import hfft ; return hfft(x)", (100*numpy.random.random(128)).astype(numpy.int64), test_hfft_int64=[NDArray[numpy.int64,:]])

    def test_hfft_byte(self):
        self.run_test("def test_hfft_byte(x): from numpy.fft import hfft ; return hfft(x)", (100*numpy.random.random(128)).astype(numpy.byte), test_hfft_byte=[NDArray[numpy.byte,:]])

@TestEnv.module
class TestNumpyFFT(TestEnv):
    # complex inputs
    def test_fft_1d_1(self):
        self.run_test("def test_fft_1d_1(x): from numpy.fft import fft ; return fft(x)", numpy.random.randn(10)+1j*numpy.random.randn(10), test_fft_1d_1=[NDArray[numpy.complex128, :]])
    # check axis arg
    def test_fft_1d_axis(self):
        axs = [0, -1]
        for ax in axs:
            with self.subTest():
                self.run_test("def test_fft_1d_axis(x, a): from numpy.fft import fft ; return fft(x, axis=a)", numpy.random.randn(10)+1j*numpy.random.randn(10), ax, test_fft_1d_axis=[NDArray[numpy.complex128, :], int])
    # check n arg set to None explicitely
    def test_fft_1d_n_None(self):
        self.run_test("def test_fft_1d_n_None(x): from numpy.fft import fft ; return fft(x, n=None)", numpy.random.randn(10)+1j*numpy.random.randn(10), test_fft_1d_n_None=[NDArray[numpy.complex128, :]])
    # check n set smaller, same and larger than axis
    def test_fft_1d_n(self):
        ns = [8, 10, 12]
        for n in ns:
            with self.subTest():
                self.run_test("def test_fft_1d_n(x, n): from numpy.fft import fft ; return fft(x, n)", numpy.random.randn(10)+1j*numpy.random.randn(10), n, test_fft_1d_n=[NDArray[numpy.complex128, :], int])
    # check norm arg
    def test_fft_1d_norm(self):
        self.run_test("def test_fft_1d_norm(x, norm): from numpy.fft import fft ; return fft(x, norm=norm)", numpy.random.randn(10)+1j*numpy.random.randn(10), "ortho", test_fft_1d_norm=[NDArray[numpy.complex128, :], str])

    ## do the same checks for real inputs
    def test_fft_1d_f64_1(self):
        self.run_test("def test_fft_1d_f64_1(x): from numpy.fft import fft ; return fft(x)", numpy.random.randn(10), test_fft_1d_f64_1=[NDArray[numpy.float64, :]])

    def test_fft_1d_real_f32(self):
        self.run_test("def test_fft_1d_real_f32(x): from numpy.fft import fft ; return fft(x)", numpy.random.randn(10).astype(numpy.float32), test_fft_1d_real_f32=[NDArray[numpy.float32, :]])

    def test_fft_1d_f64_axis(self):
        axs = [0, -1]
        for ax in axs:
            with self.subTest():
                self.run_test("def test_fft_1d_f64_axis(x, a): from numpy.fft import fft ; return fft(x, axis=a)", numpy.random.randn(10), ax, test_fft_1d_f64_axis=[NDArray[numpy.float64, :], int])

    def test_fft_1d_f64_n_None(self):
        self.run_test("def test_fft_1d_f64_n_None(x): from numpy.fft import fft ; return fft(x, n=None)", numpy.random.randn(10), test_fft_1d_f64_n_None=[NDArray[numpy.float64, :]])

    # check n set smaller, same and larger than axis
    def test_fft_1d_f64_n(self):
        ns = [8, 10, 12]
        for n in ns:
            with self.subTest():
                self.run_test("def test_fft_1d_f64_n(x, n): from numpy.fft import fft ; return fft(x, n)", numpy.random.randn(10), n, test_fft_1d_f64_n=[NDArray[numpy.float64, :], int])

    def test_fft_1d_f64_norm(self):
        self.run_test("def test_fft_1d_f64_norm(x, n): from numpy.fft import fft ; return fft(x, norm=n)", numpy.random.randn(10), "ortho", test_fft_1d_f64_norm=[NDArray[numpy.float64, :], str])

    def test_fft_1d_f64_norm_None(self):
        self.run_test("def test_fft_1d_norm_None(x): from numpy.fft import fft ; return fft(x, norm=None)", numpy.random.randn(10), test_fft_1d_norm_None=[NDArray[numpy.float64, :]])
    # check for integer intputs
    def test_fft_1d_int64(self):
        self.run_test("def test_fft_1d_int64(x): from numpy.fft import fft ; return fft(x)", numpy.random.randn(10).astype(numpy.int64), test_fft_1d_int64=[NDArray[numpy.int64, :]])

    def test_fft_1d_int32(self):
        self.run_test("def test_fft_1d_int32(x): from numpy.fft import fft ; return fft(x)", numpy.random.randn(10).astype(numpy.int32), test_fft_1d_int32=[NDArray[numpy.int32, :]])

    def test_fft_1d_int8(self):
        self.run_test("def test_fft_1d_int8(x): from numpy.fft import fft ; return fft(x)", numpy.random.randn(10).astype(numpy.int8), test_fft_1d_int8=[NDArray[numpy.int8, :]])

    def test_fft_1d_byte(self):
        self.run_test("def test_fft_1d_byte(x): from numpy.fft import fft ; return fft(x)", numpy.random.randn(10).astype(numpy.int8), test_fft_1d_byte=[NDArray[numpy.byte, :]])

    def test_fft_1d_int64_axis(self):
        axs = [0, -1]
        for ax in axs:
            with self.subTest():
                self.run_test("def test_fft_1d_int64_axis(x, a): from numpy.fft import fft ; return fft(x, axis=a)", numpy.random.randn(10).astype(numpy.int64), ax, test_fft_1d_int64_axis=[NDArray[numpy.int64, :], int])

    def test_fft_1d_int64_n_None(self):
        self.run_test("def test_fft_1d_int64_n_None(x): from numpy.fft import fft ; return fft(x, n=None)", numpy.random.randn(10).astype(numpy.int64), test_fft_1d_int64_n_None=[NDArray[numpy.int64, :]])

    # check n set smaller, same and larger than axis
    def test_fft_1d_int64_n(self):
        ns = [8, 10, 12]
        for n in ns:
            with self.subTest():
                self.run_test("def test_fft_1d_int64_n(x, n): from numpy.fft import fft ; return fft(x, n)", numpy.random.randn(10).astype(numpy.int64), n, test_fft_1d_int64_n=[NDArray[numpy.int64, :], int])

    def test_fft_1d_int64_axis(self):
        self.run_test("def test_fft_1d_int64_axis(x, a): from numpy.fft import fft ; return fft(x, axis=a)", numpy.random.randn(10).astype(numpy.int64), 0, test_fft_1d_int64_axis=[NDArray[numpy.int64, :], int])

    def test_fft_1d_int64_norm(self):
        self.run_test("def test_fft_1d_int64_norm(x, n): from numpy.fft import fft ; return fft(x, norm=n)", numpy.random.randn(10).astype(numpy.int64), "ortho", test_fft_1d_int64_norm=[NDArray[numpy.int64, :], str])

    def test_fft_1d_int64_norm_None(self):
        self.run_test("def test_fft_1d_int64_norm_None(x): from numpy.fft import fft ; return fft(x, norm=None)", numpy.random.randn(10).astype(numpy.int64), test_fft_1d_int64_norm_None=[NDArray[numpy.int64, :]])

    def test_fft_1d_2(self):
        self.run_test("def test_fft_1d_2(x): from numpy.fft import fft ; return fft(x)", numpy.random.randn(2**16)+1j*numpy.random.randn(2**16), test_fft_1d_2=[NDArray[numpy.complex128, :]])

    def test_fft_2d(self):
        szs  = [3, 5]
        for sz in szs:
            with self.subTest():
                self.run_test("def test_fft_2d_2(x): from numpy.fft import fft ; return fft(x)", (numpy.random.randn(30)+1j*numpy.random.randn(30)).reshape(sz, -1), test_fft_2d_2=[NDArray[numpy.complex128, :, :]])

    def test_fft_2d_axis_n_norm(self):
        al = [0, 1, -1]
        nl = [3, 4, 5]
        norm = "ortho"
        for a in al:
            for n in nl:
                with self.subTest():
                    self.run_test("def test_fft_2d_axis_n_norm(x, a, n, norm): from numpy.fft import fft ; return fft(x, n=n, axis=a, norm=norm)", (numpy.random.randn(20)+1j*numpy.random.randn(20)).reshape(5, -1), a, n, norm, test_fft_2d_axis_n_norm=[NDArray[numpy.complex128, :, :], int, int, str])

    def test_fft_2d_f64_axis_n_norm(self):
        al = [0, 1, -1]
        nl = [3, 4, 5]
        norm = "ortho"
        for a in al:
            for n in nl:
                with self.subTest():
                    self.run_test("def test_fft_2d_f64_axis_n_norm(x, a, n, norm): from numpy.fft import fft ; return fft(x, n=n, axis=a, norm=norm)", numpy.random.randn(20).reshape(5, -1).copy(), a, n, norm, test_fft_2d_f64_axis_n_norm=[NDArray[numpy.float64, :, :], int, int, str])

    def test_fft_2d_int64_axis_n_norm(self):
        al = [0, 1, -1]
        nl = [3, 4, 5]
        norm = "ortho"
        for a in al:
            for n in nl:
                with self.subTest():
                    self.run_test("def test_fft_2d_int64_axis_n_norm(x, a, n, norm): from numpy.fft import fft ; return fft(x, n=n, axis=a, norm=norm)", (numpy.random.randn(20)).astype(numpy.int64).reshape(5, -1).copy(), a, n, norm, test_fft_2d_int64_axis_n_norm=[NDArray[numpy.int64, :, :], int, int, str])
    # 3D
    def test_fft_3d(self):
        szs  = [(5, 4, -1), (4, 5, -1)]
        for sz in szs:
            with self.subTest():
                self.run_test("def test_fft_3d(x): from numpy.fft import fft ; return fft(x)", (numpy.random.randn(200)+1j*numpy.random.randn(200)).reshape(sz).copy(), test_fft_3d=[NDArray[numpy.complex128, :, :, :]])

    def test_fft_3d_axis(self):
        al = [0, 1, 2, -1]
        szs  = [(5, 4, -1), (4, 5, -1)]
        for a in al:
            for sz in szs:
                with self.subTest():
                    self.run_test("def test_fft_3d_axis(x, a): from numpy.fft import fft ; return fft(x, axis=a)", (numpy.random.randn(200)+1j*numpy.random.randn(200)).reshape(sz), a, test_fft_3d_axis=[NDArray[numpy.complex128, :, :, :], int])

    def test_fft_3d_f64_axis(self):
        al = [0, 1, 2, -1]
        szs  = [(5, 4, -1), (4, 5, -1)]
        for a in al:
            for sz in szs:
                with self.subTest():
                    self.run_test("def test_fft_3d_f64_axis(x, a): from numpy.fft import fft ; return fft(x, axis=a)", (numpy.random.randn(200)).reshape(sz), a, test_fft_3d_f64_axis=[NDArray[numpy.float64, :, :, :], int])

    def test_fft_3d_int64_axis(self):
        al = [0, 1, 2, -1]
        szs  = [(5, 4, -1), (4, 5, -1)]
        for a in al:
            for sz in szs:
                with self.subTest():
                    self.run_test("def test_fft_3d_int64_axis(x, a): from numpy.fft import fft ; return fft(x, axis=a)", (numpy.random.randn(200)).reshape(sz).astype(numpy.int64), a, test_fft_3d_int64_axis=[NDArray[numpy.int64, :, :, :], int])

    def test_fft_parallel(self):
        import numpy
        self.run_test("""
        import numpy as np
        def test_fft_parallel(x):
            out = [np.empty_like(x) for i in range(20)]
            #omp parallel for
            for ii in range(20):
                out[ii] = np.fft.fft(x)
            return np.concatenate(out)
        """, (numpy.random.randn(512)+1j*numpy.random.randn(512)).reshape((4,128)), test_fft_parallel=[NDArray[numpy.complex128, :, :]])

    def test_fft_f64_parallel(self):
        import numpy
        self.run_test("""
        import numpy as np
        def test_fft_f64_parallel(x):
            out = [np.empty_like(x) for i in range(20)]
            #omp parallel for
            for ii in range(20):
                out[ii] = np.fft.fft(x)
            return np.concatenate(out)
        """, (numpy.random.randn(512)).reshape((4,128)), test_fft_f64_parallel=[NDArray[numpy.float64, :, :]])

    def test_fft_int64_parallel(self):
        import numpy
        self.run_test("""
        import numpy as np
        def test_fft_int64_parallel(x):
            out = [np.empty_like(x) for i in range(20)]
            #omp parallel for
            for ii in range(20):
                out[ii] = np.fft.fft(x)
            return np.concatenate(out)
        """, (numpy.random.randn(512)).reshape((4,128)).astype(numpy.int64), test_fft_int64_parallel=[NDArray[numpy.int64, :, :]])

@TestEnv.module
class TestNumpyIFFT(TestEnv):
    # complex inputs
    def test_ifft_1d_1(self):
        self.run_test("def test_ifft_1d_1(x): from numpy.fft import ifft ; return ifft(x)", numpy.random.randn(10)+1j*numpy.random.randn(10), test_ifft_1d_1=[NDArray[numpy.complex128, :]])
    # check axis arg
    def test_ifft_1d_axis(self):
        axs = [0, -1]
        for ax in axs:
            with self.subTest():
                self.run_test("def test_ifft_1d_axis(x, a): from numpy.fft import ifft ; return ifft(x, axis=a)", numpy.random.randn(10)+1j*numpy.random.randn(10), ax, test_ifft_1d_axis=[NDArray[numpy.complex128, :], int])
    # check n arg set to None explicitely
    def test_ifft_1d_n_None(self):
        self.run_test("def test_ifft_1d_n_None(x): from numpy.fft import ifft ; return ifft(x, n=None)", numpy.random.randn(10)+1j*numpy.random.randn(10), test_ifft_1d_n_None=[NDArray[numpy.complex128, :]])
    # check n set smaller, same and larger than axis
    def test_ifft_1d_n(self):
        ns = [8, 10, 12]
        for n in ns:
            with self.subTest():
                self.run_test("def test_ifft_1d_n(x, n): from numpy.fft import ifft ; return ifft(x, n)", numpy.random.randn(10)+1j*numpy.random.randn(10), n, test_ifft_1d_n=[NDArray[numpy.complex128, :], int])
    # check norm arg
    def test_ifft_1d_norm(self):
        self.run_test("def test_ifft_1d_norm(x, norm): from numpy.fft import ifft ; return ifft(x, norm=norm)", numpy.random.randn(10)+1j*numpy.random.randn(10), "ortho", test_ifft_1d_norm=[NDArray[numpy.complex128, :], str])

    ## do the same checks for real inputs
    def test_ifft_1d_f64_1(self):
        self.run_test("def test_ifft_1d_f64_1(x): from numpy.fft import ifft ; return ifft(x)", numpy.random.randn(10), test_ifft_1d_f64_1=[NDArray[numpy.float64, :]])

    def test_ifft_1d_real_f32(self):
        self.run_test("def test_ifft_1d_real_f32(x): from numpy.fft import ifft ; return ifft(x)", numpy.random.randn(10).astype(numpy.float32), test_ifft_1d_real_f32=[NDArray[numpy.float32, :]])

    def test_ifft_1d_f64_axis(self):
        axs = [0, -1]
        for ax in axs:
            with self.subTest():
                self.run_test("def test_ifft_1d_f64_axis(x, a): from numpy.fft import ifft ; return ifft(x, axis=a)", numpy.random.randn(10), ax, test_ifft_1d_f64_axis=[NDArray[numpy.float64, :], int])

    def test_ifft_1d_f64_n_None(self):
        self.run_test("def test_ifft_1d_f64_n_None(x): from numpy.fft import ifft ; return ifft(x, n=None)", numpy.random.randn(10), test_ifft_1d_f64_n_None=[NDArray[numpy.float64, :]])

    # check n set smaller, same and larger than axis
    def test_ifft_1d_f64_n(self):
        ns = [8, 10, 12]
        for n in ns:
            with self.subTest():
                self.run_test("def test_ifft_1d_f64_n(x, n): from numpy.fft import ifft ; return ifft(x, n)", numpy.random.randn(10), n, test_ifft_1d_f64_n=[NDArray[numpy.float64, :], int])

    def test_ifft_1d_f64_norm(self):
        self.run_test("def test_ifft_1d_f64_norm(x, n): from numpy.fft import ifft ; return ifft(x, norm=n)", numpy.random.randn(10), "ortho", test_ifft_1d_f64_norm=[NDArray[numpy.float64, :], str])

    def test_ifft_1d_f64_norm_None(self):
        self.run_test("def test_ifft_1d_norm_None(x): from numpy.fft import ifft ; return ifft(x, norm=None)", numpy.random.randn(10), test_ifft_1d_norm_None=[NDArray[numpy.float64, :]])
    # check for integer intputs
    def test_ifft_1d_int64(self):
        self.run_test("def test_ifft_1d_int64(x): from numpy.fft import ifft ; return ifft(x)", numpy.random.randn(10).astype(numpy.int64), test_ifft_1d_int64=[NDArray[numpy.int64, :]])

    def test_ifft_1d_int32(self):
        self.run_test("def test_ifft_1d_int32(x): from numpy.fft import ifft ; return ifft(x)", numpy.random.randn(10).astype(numpy.int32), test_ifft_1d_int32=[NDArray[numpy.int32, :]])

    def test_ifft_1d_int8(self):
        self.run_test("def test_ifft_1d_int8(x): from numpy.fft import ifft ; return ifft(x)", numpy.random.randn(10).astype(numpy.int8), test_ifft_1d_int8=[NDArray[numpy.int8, :]])

    def test_ifft_1d_byte(self):
        self.run_test("def test_ifft_1d_byte(x): from numpy.fft import ifft ; return ifft(x)", numpy.random.randn(10).astype(numpy.int8), test_ifft_1d_byte=[NDArray[numpy.byte, :]])
        
    def test_ifft_1d_int64_axis(self):
        axs = [0, -1]
        for ax in axs:
            with self.subTest():
                self.run_test("def test_ifft_1d_int64_axis(x, a): from numpy.fft import ifft ; return ifft(x, axis=a)", numpy.random.randn(10).astype(numpy.int64), ax, test_ifft_1d_int64_axis=[NDArray[numpy.int64, :], int])

    def test_ifft_1d_int64_n_None(self):
        self.run_test("def test_ifft_1d_int64_n_None(x): from numpy.fft import ifft ; return ifft(x, n=None)", numpy.random.randn(10).astype(numpy.int64), test_ifft_1d_int64_n_None=[NDArray[numpy.int64, :]])

    # check n set smaller, same and larger than axis
    def test_ifft_1d_int64_n(self):
        ns = [8, 10, 12]
        for n in ns:
            with self.subTest():
                self.run_test("def test_ifft_1d_int64_n(x, n): from numpy.fft import ifft ; return ifft(x, n)", numpy.random.randn(10).astype(numpy.int64), n, test_ifft_1d_int64_n=[NDArray[numpy.int64, :], int])

    def test_ifft_1d_int64_axis(self):
        self.run_test("def test_ifft_1d_int64_axis(x, a): from numpy.fft import ifft ; return ifft(x, axis=a)", numpy.random.randn(10).astype(numpy.int64), 0, test_ifft_1d_int64_axis=[NDArray[numpy.int64, :], int])
        
    def test_ifft_1d_int64_norm(self):
        self.run_test("def test_ifft_1d_int64_norm(x, n): from numpy.fft import ifft ; return ifft(x, norm=n)", numpy.random.randn(10).astype(numpy.int64), "ortho", test_ifft_1d_int64_norm=[NDArray[numpy.int64, :], str])

    def test_ifft_1d_int64_norm_None(self):
        self.run_test("def test_ifft_1d_int64_norm_None(x): from numpy.fft import ifft ; return ifft(x, norm=None)", numpy.random.randn(10).astype(numpy.int64), test_ifft_1d_int64_norm_None=[NDArray[numpy.int64, :]])

    def test_ifft_1d_2(self):
        self.run_test("def test_ifft_1d_2(x): from numpy.fft import ifft ; return ifft(x)", numpy.random.randn(2**16)+1j*numpy.random.randn(2**16), test_ifft_1d_2=[NDArray[numpy.complex128, :]])

    def test_ifft_2d(self):
        szs  = [3, 5]
        for sz in szs:
            with self.subTest():
                self.run_test("def test_ifft_2d_2(x): from numpy.fft import ifft ; return ifft(x)", (numpy.random.randn(30)+1j*numpy.random.randn(30)).reshape(sz, -1), test_ifft_2d_2=[NDArray[numpy.complex128, :, :]])

    def test_ifft_2d_axis_n_norm(self):
        al = [0, 1, -1]
        nl = [3, 4, 5]
        norm = "ortho"
        for a in al:
            for n in nl:
                with self.subTest():
                    self.run_test("def test_ifft_2d_axis_n_norm(x, a, n, norm): from numpy.fft import ifft ; return ifft(x, n=n, axis=a, norm=norm)", (numpy.random.randn(20)+1j*numpy.random.randn(20)).reshape(5, -1), a, n, norm, test_ifft_2d_axis_n_norm=[NDArray[numpy.complex128, :, :], int, int, str])

    def test_ifft_2d_f64_axis_n_norm(self):
        al = [0, 1, -1]
        nl = [3, 4, 5]
        norm = "ortho"
        for a in al:
            for n in nl:
                with self.subTest():
                    self.run_test("def test_ifft_2d_f64_axis_n_norm(x, a, n, norm): from numpy.fft import ifft ; return ifft(x, n=n, axis=a, norm=norm)", numpy.random.randn(20).reshape(5, -1).copy(), a, n, norm, test_ifft_2d_f64_axis_n_norm=[NDArray[numpy.float64, :, :], int, int, str])

    def test_ifft_2d_int64_axis_n_norm(self):
        al = [0, 1, -1]
        nl = [3, 4, 5]
        norm = "ortho"
        for a in al:
            for n in nl:
                with self.subTest():
                    self.run_test("def test_ifft_2d_int64_axis_n_norm(x, a, n, norm): from numpy.fft import ifft ; return ifft(x, n=n, axis=a, norm=norm)", (numpy.random.randn(20)).astype(numpy.int64).reshape(5, -1).copy(), a, n, norm, test_ifft_2d_int64_axis_n_norm=[NDArray[numpy.int64, :, :], int, int, str])
    # 3D
    def test_ifft_3d(self):
        szs  = [(5, 4, -1), (4, 5, -1)]
        for sz in szs:
            with self.subTest():
                self.run_test("def test_ifft_3d(x): from numpy.fft import ifft ; return ifft(x)", (numpy.random.randn(200)+1j*numpy.random.randn(200)).reshape(sz).copy(), test_ifft_3d=[NDArray[numpy.complex128, :, :, :]])
                
    def test_ifft_3d_axis(self):
        al = [0, 1, 2, -1]
        szs  = [(5, 4, -1), (4, 5, -1)]
        for a in al:
            for sz in szs:
                with self.subTest():
                    self.run_test("def test_ifft_3d_axis(x, a): from numpy.fft import ifft ; return ifft(x, axis=a)", (numpy.random.randn(200)+1j*numpy.random.randn(200)).reshape(sz), a, test_ifft_3d_axis=[NDArray[numpy.complex128, :, :, :], int])

    def test_ifft_3d_f64_axis(self):
        al = [0, 1, 2, -1]
        szs  = [(5, 4, -1), (4, 5, -1)]
        for a in al:
            for sz in szs:
                with self.subTest():
                    self.run_test("def test_ifft_3d_f64_axis(x, a): from numpy.fft import ifft ; return ifft(x, axis=a)", (numpy.random.randn(200)).reshape(sz), a, test_ifft_3d_f64_axis=[NDArray[numpy.float64, :, :, :], int])

    def test_ifft_3d_int64_axis(self):
        al = [0, 1, 2, -1]
        szs  = [(5, 4, -1), (4, 5, -1)]
        for a in al:
            for sz in szs:
                with self.subTest():
                    self.run_test("def test_ifft_3d_int64_axis(x, a): from numpy.fft import ifft ; return ifft(x, axis=a)", (numpy.random.randn(200)).reshape(sz).astype(numpy.int64), a, test_ifft_3d_int64_axis=[NDArray[numpy.int64, :, :, :], int])

    def test_ifft_parallel(self):
        import numpy
        self.run_test("""
        import numpy as np
        def test_ifft_parallel(x):
            out = [np.empty_like(x) for i in range(20)]
            #omp parallel for
            for ii in range(20):
                out[ii] = np.fft.fft(x)
            return np.concatenate(out)
        """, (numpy.random.randn(512)+1j*numpy.random.randn(512)).reshape((4,128)), test_ifft_parallel=[NDArray[numpy.complex128, :, :]])
        
    def test_ifft_f64_parallel(self):
        import numpy
        self.run_test("""
        import numpy as np
        def test_ifft_f64_parallel(x):
            out = [np.empty_like(x) for i in range(20)]
            #omp parallel for
            for ii in range(20):
                out[ii] = np.fft.fft(x)
            return np.concatenate(out)
        """, (numpy.random.randn(512)).reshape((4,128)), test_ifft_f64_parallel=[NDArray[numpy.float64, :, :]])

    def test_ifft_int64_parallel(self):
        import numpy
        self.run_test("""
        import numpy as np
        def test_ifft_int64_parallel(x):
            out = [np.empty_like(x) for i in range(20)]
            #omp parallel for
            for ii in range(20):
                out[ii] = np.fft.fft(x)
            return np.concatenate(out)
        """, (numpy.random.randn(512)).reshape((4,128)).astype(numpy.int64), test_ifft_int64_parallel=[NDArray[numpy.int64, :, :]])



#from http://rosettacode.org/wiki/Gray_code
#pythran export bin2gray(int list)
#pythran export gray2bin(int list)
#pythran export int2bin(int)
#pythran export bin2int(int list)
#runas [int2bin(i) for i in range(16)]
#runas [bin2gray(int2bin(i)) for i in range(16)]
#runas [gray2bin(bin2gray(int2bin(i))) for i in range(16)]
#runas [bin2int(gray2bin(bin2gray(int2bin(i)))) for i in range(16)]

def bin2gray(bits):
    return bits[:1] + [i ^ ishift for i, ishift in zip(bits[:-1], bits[1:])]

def gray2bin(bits):
    b = [bits[0]]
    for nextb in bits[1:]: b.append(b[-1] ^ nextb)
    return b


def int2bin(n):
    'From positive integer to list of binary bits, msb at index 0'
    if n:
        bits = []
        while n:
            n,remainder = divmod(n, 2)
            bits.insert(0, remainder)
        return bits
    else: return [0]


def bin2int(bits):
    'From binary bits, msb at index 0 to integer'
    i = 0
    for bit in bits:
        i = i * 2 + bit
    return i



#adapted from http://www.daniweb.com/software-development/python/code/216689/sorting-algorithms-in-python
#pythran export bubble_sort(int list)
#runas bubble_sort([-4,1,45,-6,123,4,6,1,34,-8,12])
#bench import random; in_ = random.sample(range(1000000000), 4000); bubble_sort(in_)
def bubble_sort(list0):
    list1=[x for x in list0 ] # simulate copy
    for i in range(0, len(list1) - 1):
        swap_test = False
        for j in range(0, len(list1) - i - 1):
            if list1[j] > list1[j + 1]:
                list1[j], list1[j + 1] = list1[j + 1], list1[j]  # swap
            swap_test = True
        if swap_test == False:
            break
    return list1



#from http://www.emoticode.net/python/python-implementation-of-rc4-algorithm.html
#runas data = "e"*100 ; key = "f"*3 ; rc4_crypt(data, key)
#bench data = "e"*2000000 ; key = "f"*3 ; rc4_crypt(data, key)
#pythran export rc4_crypt(str, str)
#unittest.python3.skip

#RC4 Implementation
def rc4_crypt( data , key ):

    S = range(256)
    j = 0
    out = []

    #KSA Phase
    for i in range(256):
        j = (j + S[i] + ord( key[i % len(key)] )) % 256
        S[i] , S[j] = S[j] , S[i]

    #PRGA Phase
    for char in data:
        i = j = 0
        i = (i + 1) % 256
        j = (j + S[i]) % 256
        S[i], S[j] = S[j], S[i]
        out.append(chr(ord(char) ^ S[(S[i] + S[j]) % 256]))

    return ''.join(out)



import re

import pythran
from pythran.tests import TestEnv


class TestVersion(TestEnv):
    def test_version_check_cython(self):
        # Cython actually does this check (variable is named
        # `pythran_is_pre_0_9_6`). Make sure it doesn't break.
        v = pythran.__version__
        pre_096 = tuple(map(int, v.split('.')[0:3])) < (0, 9, 6)
        self.assertFalse(pre_096)

    def test_valid_version_string(self):
        # Verify that the pythran version is a valid one (note: excludes
        # .post suffix, and complies to PEP 440. Test taken from NumPy
        version_pattern = r"^[0-9]+\.[0-9]+\.[0-9]+(|a[0-9]|b[0-9]|rc[0-9])"
        dev_suffix = r"\.dev0\+[0-9]*\.g[0-9a-f]+"

        # For released versions:
        res1 = re.match(version_pattern, pythran.__version__)
        # For dev versions:
        res2 = re.match(version_pattern + dev_suffix, pythran.__version__)

        self.assertTrue(res1 is not None or res2 is not None,
                        pythran.__version__)




def omp_parallel_default():
    import omp
    sum = 0
    known_sum = (1000 * (1000 + 1)) / 2

    if "omp parallel default(shared)":
        mysum = 0
        'omp for'
        for i in range(1, 1001):
            mysum += i

        if 'omp critical':
            sum += mysum

    return known_sum == sum



#from http://rosettacode.org/wiki/Read_a_configuration_file#Python
#pythran export readconf(str)
#runas readconf("pythran/tests/rosetta/read_conf.cfg")

def readconf(fn):
    ret = {}
    fp = open(fn)
    for line in fp:
        # Assume whitespace is ignorable
        line = line.strip()
        if not line or line.startswith('#'): continue

        boolval = "True"
        # Assume leading ";" means a false boolean
        if line.startswith(';'):
            # Remove one or more leading semicolons
            line = line.lstrip(';')
            # If more than just one word, not a valid boolean
            if len(line.split()) != 1: continue
            boolval = "False"

        bits = line.split(None, 1)
        if len(bits) == 1:
            # Assume booleans are just one standalone word
            k = bits[0]
            v = boolval
        else:
            # Assume more than one word is a string value
            k, v = bits
        ret[k.lower()] = v
    fp.close()
    return ret



# Author: Pim Schellart
# 2010 - 2011

"""Tools for spectral analysis of unequally sampled signals."""

import numpy as np
#pythran export lombscargle(float64[], float64[], float64[])
#runas import numpy; x = numpy.arange(2., 12.); y = numpy.arange(1., 11.); z = numpy.arange(3., 13.); lombscargle(x, y, z)

def lombscargle(x, y, freqs):
    """
    _lombscargle(x, y, freqs)

    Computes the Lomb-Scargle periodogram.
    
    Parameters
    ----------
    x : array_like
        Sample times.
    y : array_like
        Measurement values (must be registered so the mean is zero).
    freqs : array_like
        Angular frequencies for output periodogram.

    Returns
    -------
    pgram : array_like
        Lomb-Scargle periodogram.

    Raises
    ------
    ValueError
        If the input arrays `x` and `y` do not have the same shape.

    See also
    --------
    lombscargle

    """

    # Check input sizes
    if x.shape != y.shape:
        raise ValueError("Input arrays do not have the same size.")

    # Local variables
    c = np.cos(freqs[:, None] * x)
    s = np.sin(freqs[:, None] * x)
    xc = np.sum(y * c, axis=1)
    xs = np.sum(y * s, axis=1)
    cc = np.sum(c ** 2, axis=1)
    ss = np.sum(s * s, axis=1)
    cs = np.sum(c * s, axis=1)
    tau = np.arctan2(2 * cs, cc - ss) / (2 * freqs)
    c_tau = np.cos(freqs * tau)
    s_tau = np.sin(freqs * tau)
    c_tau2 = c_tau * c_tau
    s_tau2 = s_tau * s_tau
    cs_tau = 2 * c_tau * s_tau

    pgram = 0.5 * (((c_tau * xc + s_tau * xs)**2 / \
        (c_tau2 * cc + cs_tau * cs + s_tau2 * ss)) + \
        ((c_tau * xs - s_tau * xc)**2 / \
        (c_tau2 * ss - cs_tau * cs + s_tau2 * cc)))



    return pgram




#from http://rosettacode.org/wiki/Average_loop_length#Python
#pythran export analytical(int)
#pythran export testing(int, int)
#runas analytical(5)
#runas avg = testing(10, 10**5); theory = analytical(10); abs((avg / theory - 1)) < 0.01

from math import factorial
from random import randrange

def analytical(n):
    return sum(factorial(n) / pow(n, i) / float(factorial(n - i))
               for i in range(1, n+1))

def testing(n, times):
    count = 0
    for i in range(times):
        x, bits = 1, 0
        while not (bits & x):
            count += 1
            bits |= x
            x = 1 << randrange(n)
    return count / times



#runas import numpy as np; x = np.arange(1., 26.).reshape(5,5); factorMatrix0(x), factorMatrix1(x)

import numpy as np
#pythran export factorMatrix0(float[:,:])
def factorMatrix0(M):
# Gaussian elimination, partial pivoting.
# M must be an (n,n+1) numpy array. Not tested!
    n = M.shape[0]
    m=  M.shape[1]
    for line in range(0, n-1):
        # find pivot
        cmax = np.argmax(abs(M[line:n,line])) + line

        # exchange rows if necessary
        if cmax != line:
            M[[line,cmax]]=M[[cmax,line]]

        # eliminate
        pivot = M[line,line]
        for j in range(line+1,n):
            v= M[j,line]/pivot
            for k in range(line,m):
                M[j,k]-= v*M[line,k]

#pythran export factorMatrix1(float[:,:])
def factorMatrix1(M):
# Gaussian elimination, partial pivoting.
# M must be an (n,n+1) numpy array. Not tested!
    n = M.shape[0]
    m=  M.shape[1]
    for line in range(0, n-1):
        # find pivot
        cmax=line
        vmax= abs(M[line,line])
        for i in range(line+1,n):
            if abs(M[i,line])> vmax:
                vmax= abs(M[i,line])
                cmax= i
        # exchange rows if necessary
        if cmax != line:
            for j in range(line,m):
                t= M[line,j]
                M[line,j]= M[cmax,j]
                M[cmax,j]= t
        # eliminate
        pivot = M[line,line]
        for j in range(line+1,n):
            v= M[j,line]/pivot
            for k in range(line,m):
                M[j,k]-= v*M[line,k]



#pythran export laplacian(float [], float)
#runas import numpy as np; from lap import laplacian; var = np.arange(100, dtype=float) ; dh2=.01 ; laplacian(var, dh2)
#from http://stackoverflow.com/questions/32485935/cythonize-a-partial-differential-equation-integrator
import numpy
def laplacian(var, dh2):
    """ (1D array, dx^2) -> laplacian(1D array)
    periodic_laplacian_1D_4th_order
    Implementing the 4th order 1D laplacian with periodic condition
    """
    lap = numpy.zeros_like(var)
    lap[1:]    = (4.0/3.0)*var[:-1]
    lap[0]     = (4.0/3.0)*var[1]
    lap[:-1]  += (4.0/3.0)*var[1:]
    lap[-1]   += (4.0/3.0)*var[0]
    lap       += (-5.0/2.0)*var

    lap[2:]   += (-1.0/12.0)*var[:-2]
    lap[:2]   += (-1.0/12.0)*var[-2:]
    lap[:-2]  += (-1.0/12.0)*var[2:]
    lap[-2:]  += (-1.0/12.0)*var[:2]

    return lap / dh2



#from http://stackoverflow.com/questions/19277244/fast-weighted-euclidean-distance-between-points-in-arrays/19277334#19277334
#pythran export slow_wdist(float64 [][], float64 [][], float64[][])
#runas import numpy as np ; A = np.arange(6.).reshape((2,3)) ; B =  np.arange(1,7.).reshape((2,3)) ; W = np.arange(2,8.).reshape((2,3)) ; slow_wdist(A,B,W)
#bench S = 520.; import numpy as np ; A = np.arange(S).reshape((2,S / 2)) ; B =  np.arange(1,1 + S).reshape((2,S / 2)) ; W = np.arange(2,S + 2).reshape((2,S / 2)) ; slow_wdist(A,B,W)

import numpy as np
def slow_wdist(A, B, W):

    k,m = A.shape
    _,n = B.shape
    D = np.zeros((m, n))

    for ii in range(m):
        for jj in range(n):
            wdiff = (A[:,ii] - B[:,jj]) / W[:,ii]
            D[ii,jj] = np.sqrt((wdiff**2).sum())
    return D



#from http://rosettacode.org/wiki/Range_expansion#Python
#pythran export rangeexpand(str)
#runas rangeexpand('-6,-3--1,3-5,7-11,14,15,17-20')

def rangeexpand(txt):
    lst = []
    for r in txt.split(','):
        if '-' in r[1:]:
            r0, r1 = r[1:].split('-', 1)
            lst += range(int(r[0] + r0), int(r1) + 1)
        else:
            lst.append(int(r))
    return lst



class FunMeta(type):

    def __getitem__(cls, item):
        return Fun(tuple(item[0]) + (item[1],))


class DictMeta(type):

    def __getitem__(cls, item):
        return Dict(item)


class UnionMeta(type):

    def __getitem__(cls, item):
        return Union(item)


class SetMeta(type):

    def __getitem__(cls, item):
        return Set(item)


class ListMeta(type):

    def __getitem__(cls, item):
        return List(item)


class IterableMeta(type):

    def __getitem__(cls, item):
        return Iterable(item)


class GeneratorMeta(type):

    def __getitem__(cls, item):
        return Generator(item)


class TupleMeta(type):

    def __getitem__(cls, item):
        return Tuple(item)


class OptionalMeta(type):

    def __getitem__(cls, item):
        return Optional(item)


class NDArrayMeta(type):

    def __getitem__(cls, item):
        return NDArray(item)


class PointerMeta(type):

    def __getitem__(cls, item):
        return Pointer(item)


class Type(type):

    def __new__(cls, args):
        return type.__new__(
            cls,
            cls.__name__,
            (object,),
            {'__args__': args if isinstance(args, tuple) else (args,)}
        )

    def __init__(self, *args, **kwargs):
        pass


class Fun(Type, metaclass=FunMeta):
    pass


class Dict(Type, metaclass=DictMeta):
    pass


class Union(Type, metaclass=UnionMeta):
    pass


class Set(Type, metaclass=SetMeta):
    pass


class List(Type, metaclass=ListMeta):
    pass


class Iterable(Type, metaclass=IterableMeta):
    pass


class Generator(Type, metaclass=GeneratorMeta):
    pass


class Tuple(Type, metaclass=TupleMeta):
    pass


class Optional(Type, metaclass=OptionalMeta):
    pass


class NDArray(Type, metaclass=NDArrayMeta):
    pass


class Pointer(Type, metaclass=PointerMeta):
    pass


class TypeVar(object):

    def __init__(self, name):
        self.__name__ = name


class Sized(object):
    pass


class Any(object):
    pass


class File(object):
    pass



__all__ = ['add_pressure_diffusion', 'cross1', 'cross2']

#runas import numpy as np; x = np.empty((3,2,5,7)); y = np.arange(210.).reshape(3,2,5,7); z = np.arange(210.).reshape(3,2,5,7) + 3; cross1(x,y,z)
#runas import numpy as np; x = np.empty((3,2,5,7), dtype=complex); y = np.arange(210.).reshape(3,2,5,7); z = np.arange(210., dtype=complex).reshape(3,2,5,7) + 3; cross2(x,y,z)
#runas import numpy as np; x = np.empty((3,2,5,7), dtype=complex); y = np.arange(210, dtype=np.int64).reshape(3,2,5,7); z = np.arange(210., dtype=complex).reshape(3,2,5,7) + 3; cross2(x,y,z)
#runas import numpy as np; x = np.ones((3,2,5,7), dtype=complex); y = np.arange(210, dtype=complex).reshape(3,2,5,7); z = np.arange(70.).reshape(2,5,7); w = np.arange(210.).reshape(3,2,5,7) + 3; t = np.arange(70., dtype=complex).reshape(2,5, 7) + 1 ; u = np.arange(210.).reshape(3,2,5,7) + 8; add_pressure_diffusion(x,y,z,w,t,u,3.)

#pythran export cross1(float[:,:,:,:], float[:,:,:,:], float[:,:,:,:])
def cross1(c, a, b):
    c[0] = a[0] * b[2] - a[2] * b[1]
    c[1] = a[2] * b[0] - a[0] * b[2]
    c[2] = a[0] * b[1] - a[1] * b[0]
    return c

#pythran export cross2(complex[:,:,:,:], float[:,:,:,:], complex[:,:,:,:])
#pythran export cross2(complex[:,:,:,:], int64[:,:,:,:], complex[:,:,:,:])
def cross2(c, a, b):
    cross1(c, a, b)
    c *= 1j
    return c

#pythran export add_pressure_diffusion(
#    complex[:,:,:,:], complex[:,:,:,:],
#    float[:,:,:], float[:,:,:,:], complex[:,:,:],
#    float[:,:,:,:], float)
def add_pressure_diffusion(dU, U_hat, K2, K, P_hat, K_over_K2, nu):
    du0, du1, du2 = dU[0], dU[1], dU[2]
    k_0, k_1, k_2 = K_over_K2[0], K_over_K2[1], K_over_K2[2]
    P_hat[:] = du0*k_0+du1*k_1+du2*k_2
    dU[:] = dU - P_hat*K - nu*K2*U_hat
    return P_hat



from pythran.tests import TestEnv
from textwrap import dedent
import pythran

class TestImportAll(TestEnv):

    def test_import_all(self):
        self.run_test("from math import *\ndef import_all(l): return cos(l)", 3.3, import_all=[float])

    def test_import_cmath_all(self):
        self.run_test("from cmath import *\ndef import_cmath_all(l): return cos(l)", 2.2, import_cmath_all=[float])

    def test_import_all_cos(self):
        self.run_test("from math import *\nfrom math import cos\ndef import_all_cos(l): return cos(l)", 1.1, import_all_cos=[float])

    def test_import_all_twice(self):
        self.run_test("from math import *\nfrom math import *\ndef import_all_twice(l): return cos(l)", 0.1, import_all_twice=[float])

    def test_import_same_name(self):
        self.run_test("from math import *\ndef cos(l): return 100", 0.1, cos=[float])

    def test_import_collections(self):
        """
        Check correct error is returned for incorrect module import.

        Check is done for module as .py file.
        """
        code = """
            import collections
            def unsupported_module():
                return collections.Counter()"""

        with self.assertRaises(pythran.syntax.PythranSyntaxError) as ex:
            pythran.compile_pythrancode("import_collections", dedent(code), pyonly=True)

        self.assertIn("Module 'collections' not found.", str(ex.exception))

    def test_complex_import_manipulation0(self):
        """
        Check correct error is returned for incorrect module manipulation.
        """
        code = """
            import math
            def unsupported_module():
                return math"""

        with self.assertRaises(pythran.syntax.PythranSyntaxError) as ex:
            pythran.compile_pythrancode("complex_import_manipulation0", dedent(code), pyonly=True)

    def test_complex_import_manipulation1(self):
        code = """
            import bisect
            def unsupported_module():
                return bisect()"""

        with self.assertRaises(pythran.syntax.PythranSyntaxError) as ex:
            pythran.compile_pythrancode("complex_import_manipulation1", dedent(code), pyonly=True)

    def test_complex_import_manipulation2(self):
        code = """
            from bisect import bisect_right
            def unsupported_module():
                return bisect()"""

        with self.assertRaises(pythran.syntax.PythranSyntaxError) as ex:
            pythran.compile_pythrancode("complex_import_manipulation2", dedent(code), pyonly=True)

    def test_complex_import_manipulation3(self):
        code = """
            from numpy import random
            def unsupported_module():
                return random.i_do_not_exist()"""

        with self.assertRaises(pythran.syntax.PythranSyntaxError) as ex:
            pythran.compile_pythrancode("complex_import_manipulation3", dedent(code), pyonly=True)



from distutils.core import setup
from Cython.Build import cythonize

setup(
    name = "diffuse",
    ext_modules = cythonize('diffuse.pyx'),
    script_name = 'setup.py',
    script_args = ['build_ext', '--inplace']
)

import diffuse
diffuse.test()



import unittest
from pythran.tests import TestEnv
import numpy as np

from pythran.typing import NDArray, List, Tuple


@TestEnv.module
class TestBroadcasting(TestEnv):

    def test_broadcast_scalar0(self):
        self.run_test('def broadcast_scalar0(x): return x + 1',
                      np.arange(12000),
                      broadcast_scalar0=[NDArray[int, :]])

    def test_broadcast_scalar1(self):
        self.run_test('def broadcast_scalar1(x): return x + 1',
                      np.arange(12000).reshape(40, 300),
                      broadcast_scalar1=[NDArray[int, :, :]])

    def test_broadcast_array0(self):
        self.run_test('def broadcast_array0(x, y): return x + y',
                      np.arange(12000).reshape(40,300),
                      np.arange(300),
                      broadcast_array0=[NDArray[int,:,:], NDArray[int,:]])

    def test_broadcast_array1(self):
        self.run_test('def broadcast_array1(x, y): return x + y',
                      np.arange(12000).reshape(4,30,100),
                      np.arange(100),
                      broadcast_array1=[NDArray[int,:,:,:], NDArray[int,:]])

    def test_broadcast_array2(self):
        self.run_test('def broadcast_array2(x, y): return x + y',
                      np.arange(12000).reshape(4,30,2, 50),
                      [3] * 50,
                      broadcast_array2=[NDArray[int,:,:,:,:], List[int]])

    def test_broadcast_array3(self):
        self.run_test('def broadcast_array3(x, y): return (x + y)[1:,1:,0]',
                      np.arange(12000).reshape(4,30,2, 50),
                      [3] * 50,
                      broadcast_array3=[NDArray[int,:,:,:,:], List[int]])

    def test_broadcast_with_ref(self):
        code = '''
            import numpy as np
            import numpy.linalg as npl

            def _get_cross(a, b):
                return a[0] * b[:, :, 1] - a[1] * b[:, :, 0]


            def _get_mgrid(shape):
                xx = np.empty(shape, dtype=int)
                xx.T[:] = np.arange(0, shape[0], 1, dtype=int)
                yy = np.empty(shape, dtype=int)
                yy[:] = np.arange(0, shape[1], 1, dtype=int)
                return xx, yy

            def broadcast_array_with_ref(shape, center, angle):
                xx, yy = _get_mgrid(shape)
                coords = np.rollaxis(np.array([xx, yy]), 0, 3)
                far = center + np.array([np.cos(angle), np.sin(angle)]) * 100

                if angle > 1:
                    z = np.absolute(_get_cross(far - center, coords - center))
                    return z
                else:
                    z = _get_cross(far - center, coords - center)
                    return np.absolute(z)
                '''
        self.run_test(code,
                      (3,3),
                      (4.,4.),
                      5.,
                      broadcast_array_with_ref=[Tuple[int, int], Tuple[float, float], float])


    def test_broadcast_same_dim0(self):
        self.run_test('def broadcast_same_dim0(x, y): return x + y',
                      np.arange(12000).reshape(40,300),
                      np.arange(300).reshape(1,300),
                      broadcast_same_dim0=[NDArray[int, :, :], NDArray[int, :, :]])

    def test_broadcast_same_dim1(self):
        self.run_test('def broadcast_same_dim1(x, y): return x + y',
                      np.arange(12000).reshape(40,300),
                      np.arange(300),
                      broadcast_same_dim1=[NDArray[int, :, :], NDArray[int, :]])

    def test_broadcast_both_dims(self):
        self.run_test('def broadcast_both_dims(x, y): return x + y',
                      np.arange(100).reshape(1,100),
                      np.arange(100).reshape(100, 1),
                      broadcast_both_dims=[NDArray[int, :, :], NDArray[int, :, :]])

    def test_broadcast_update_scalar0(self):
        self.run_test('def broadcast_update_scalar0(x):  x += 1; return x',
                      np.arange(12000),
                      broadcast_update_scalar0=[NDArray[int, :]])

    def test_broadcast_update_scalar1(self):
        self.run_test('def broadcast_update_scalar1(x): x += 1; return x',
                      np.arange(12000).reshape(40, 300),
                      broadcast_update_scalar1=[NDArray[int, :, :]])

    def test_broadcast_update_array0(self):
        self.run_test('def broadcast_update_array0(x, y): x += y ; return x',
                      np.arange(12000).reshape(40,300),
                      np.arange(300),
                      broadcast_update_array0=[NDArray[int, :, :], NDArray[int, :]])

    def test_broadcast_update_array1(self):
        self.run_test('def broadcast_update_array1(x, y): x += y; return x',
                      np.arange(12000).reshape(4,30,100),
                      np.arange(100),
                      broadcast_update_array1=[NDArray[int, :, :,:], NDArray[int, :]])

    def test_broadcast_update_same_dim0(self):
        self.run_test('def broadcast_update_same_dim0(x, y): x += y; return x',
                      np.arange(12000).reshape(40,300),
                      np.arange(300).reshape(1,300),
                      broadcast_update_same_dim0=[NDArray[int, :, :], NDArray[int, :, :]])

    def test_broadcast_update_same_dim1(self):
        self.run_test('def broadcast_update_same_dim1(x, y): x += y; return x',
                      np.arange(12000).reshape(40,300),
                      np.arange(300),
                      broadcast_update_same_dim1=[NDArray[int, :, :], NDArray[int, :]])

    def test_broadcast_compute_scalar0(self):
        self.run_test('def broadcast_compute_scalar0(x): return -(x + 1)',
                      np.arange(12000),
                      broadcast_compute_scalar0=[NDArray[int, :]])

    def test_broadcast_compute_scalar1(self):
        self.run_test('def broadcast_compute_scalar1(x): return -(x + 1)',
                      np.arange(12000).reshape(40, 300),
                      broadcast_compute_scalar1=[NDArray[int, :, :]])

    def test_broadcast_compute_array0(self):
        self.run_test('def broadcast_compute_array0(x, y): return -(x + y)',
                      np.arange(12000).reshape(40,300),
                      np.arange(300),
                      broadcast_compute_array0=[NDArray[int, :, :], NDArray[int, :]])

    def test_broadcast_compute_array1(self):
        self.run_test('def broadcast_compute_array1(x, y): return -(x + y)',
                      np.arange(12000).reshape(4,30,100),
                      np.arange(100),
                      broadcast_compute_array1=[NDArray[int, :, :, :], NDArray[int, :]])

    def test_broadcast_compute_same_dim0(self):
        self.run_test('def broadcast_compute_same_dim0(x, y): return -(x + y)',
                      np.arange(12000).reshape(40,300),
                      np.arange(300).reshape(1,300),
                      broadcast_compute_same_dim0=[NDArray[int, :, :], NDArray[int, :, :]])

    def test_broadcast_compute_same_dim1(self):
        self.run_test('def broadcast_compute_same_dim1(x, y): return -(x + y)',
                      np.arange(12000).reshape(40,300),
                      np.arange(300),
                      broadcast_compute_same_dim1=[NDArray[int, :, :], NDArray[int, :]])

    def test_broadcast_compute_both_dims(self):
        self.run_test('def broadcast_compute_both_dims(x, y): return -(x + y)',
                      np.arange(100).reshape(1,100),
                      np.arange(100).reshape(100, 1),
                      broadcast_compute_both_dims=[NDArray[int, :, :], NDArray[int, :, :]])

    def test_broadcast_with_reshape(self):
        self.run_test('def broadcast_with_reshape(x): n = x.shape[0]; return x.reshape(1, n) + x.reshape(n, 1)',
                      np.arange(100),
                      broadcast_with_reshape=[NDArray[int, :]])

    def test_broadcast_sum(self):
        code = '''
        def broadcast_sum(x, y):
            n = x.size
            return (x.reshape(n, 1) * y.reshape(1, n)).sum()
        '''
        self.run_test(code,
                      np.arange(100).reshape(1,100),
                      np.arange(100).reshape(100, 1),
                      broadcast_sum=[NDArray[int, :, :], NDArray[int, :, :]])

    def test_broadcast_sum_axis(self):
        code = '''
            import numpy as np
            def broadcast_sum_axis(forecasted_array, observed_array):
                return np.abs(forecasted_array[:, None] - observed_array).sum()
            '''
        self.run_test(code,
                      np.arange(100.),
                      np.arange(100.),
                      broadcast_sum_axis=[NDArray[float, :], NDArray[float, :]])

    def test_broadcasting_nth29(self):
        code = '''
        def broadcast_nth29(n):
            res = (n+2*pow(n**2,2))/n
            return res'''
        self.run_test(code,
                      np.arange(1., 100.),
                      broadcast_nth29=[NDArray[float, :]])

    def test_broadcasting_expr0(self):
        code = '''
        import numpy as np
        def broadcasting_expr0(num_pairs):
            t = np.zeros((num_pairs, 4), dtype=float)
            t[0,1] = t[1, 0] = 1
            first_row = t[:, 0].reshape((-1, 1))
            normalized_triples = np.subtract(t, first_row)
            return normalized_triples[:,1:4]'''
        self.run_test(code, 4, broadcasting_expr0=[int])

